from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S, diag, Eq,
                   hessian, Function, flatten, Tuple, im, pi, latex, dsolve,
                   solve, fraction, factorial,Subs, Number, oo, Abs, N)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ..dynamics import LagrangesDynamicSystem, HarmonicOscillator

from .elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame,base_origin
from ..continuous import ContinuousSystem, PlaneStressProblem



import base64
import random
import IPython as IP
import numpy as np

import inspect

class ComposedSystem(HarmonicOscillator):
    """Base class for all systems

    """
    scheme_name = 'damped_car_new.PNG'
    real_name = 'car_real.jpg'

    @classmethod
    def _scheme(cls):

        path = __file__.replace('continuous.py', 'images/') + cls.scheme_name

        return path

    @classmethod
    def _real_example(cls):

        path = __file__.replace('continuous.py', 'images/') + cls.real_name

        return path

    @classmethod
    def preview(cls, example=False):
        if example:
            path = cls._real_example()

        else:
            path = cls._scheme()

        with open(f"{path}", "rb") as image_file:
            encoded_string = base64.b64encode(image_file.read())
        image_file.close()

        return IP.display.Image(base64.b64decode(encoded_string))


    def get_default_data(self):
        return None

    def get_random_parameters(self):

        default_data_dict = self.get_default_data()


        
        if default_data_dict:
            parameters_dict = {
                key: random.choice(items_list)
                for key, items_list in default_data_dict.items()
            }
        else:
            parameters_dict=None

        return parameters_dict


class ContinuousSystem(ContinuousSystem):
    """Base class for all systems

    """
    scheme_name = 'damped_car_new.PNG'
    real_name = 'car_real.jpg'

    @classmethod
    def _scheme(cls):

        path = __file__.replace('continuous.py', 'images/') + cls.scheme_name

        return path

    @classmethod
    def _real_example(cls):

        path = __file__.replace('continuous.py', 'images/') + cls.real_name

        return path

    
    
    @classmethod
    def from_random_data(cls):
        
        system=cls()
        subs_dict=system.get_random_parameters()
        
        return system.subs(subs_dict)
    
    
    @classmethod
    def preview(cls, example=False):
        if example:
            path = cls._real_example()

        else:
            path = cls._scheme()

        with open(f"{path}", "rb") as image_file:
            encoded_string = base64.b64encode(image_file.read())
        image_file.close()

        return IP.display.Image(base64.b64decode(encoded_string))

    def get_default_data(self):
        return None

    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        if default_data_dict:
            parameters_dict = {
                key: random.choice(items_list)
                for key, items_list in default_data_dict.items()
            }
        else:
            parameters_dict=None

        return parameters_dict    
    
class PlaneStressProblem(PlaneStressProblem):
    """Base class for all systems

    """
    scheme_name = 'damped_car_new.PNG'
    real_name = 'car_real.jpg'

    @classmethod
    def _scheme(cls):

        path = __file__.replace('continuous.py', 'images/') + cls.scheme_name

        return path

    @classmethod
    def _real_example(cls):

        path = __file__.replace('continuous.py', 'images/') + cls.real_name

        return path

    @classmethod
    def preview(cls, example=False):
        if example:
            path = cls._real_example()

        else:
            path = cls._scheme()

        with open(f"{path}", "rb") as image_file:
            encoded_string = base64.b64encode(image_file.read())
        image_file.close()

        return IP.display.Image(base64.b64decode(encoded_string))

    def get_default_data(self):
        return None

    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        if default_data_dict:
            parameters_dict = {
                key: random.choice(items_list)
                for key, items_list in default_data_dict.items()
            }
        else:
            parameters_dict=None

        return parameters_dict    



class CSBeam(ContinuousSystem):
    """
    Model of a Continuous Beam

        Arguments:
        =========
            m = Mass
                -Mass of the payload



        Example
        =======
        A vibrating beam for given Young modulus and moment of inertia can be created in the following way.

        >>> t = symbols('t')
        >>> R,l_0 = symbols('R, l_0',positive=True)
        >>> MDoFWinch(r=R,l=l_0)

        -define the symbols and dynamicsymbols
        -determine the instance of the beam by using class CSBeam()
    """

    scheme_name = 'supported_beam_scheme.PNG'
    real_name = 'supported_beam_real.PNG'

    def __init__(self,
                E=Symbol('E',positive=True),
                A=Symbol('A',positive=True),
                I=Symbol('I',positive=True),
                rho=Symbol('\\rho',positive=True),
                w=Function('w'),
                bc_dict=None,
                time=Symbol('t'),
                loc=Symbol('x'),
                **kwargs
                ):

        self.E = E
        self.A = A
        self.I = I
        self.rho = rho
        self.w=w(time,loc)
        self.time=time
        self.loc=loc
        self.l=Symbol('L',positive=True)
        

        L_beam=S.One/2*(A*rho*(self.w.diff(self.time))**2-E*I*(self.w.diff(self.loc,2))**2)

        super().__init__(L_beam,q=self.w,bc_dict=bc_dict,t_var=self.time, spatial_var=self.loc,**kwargs)
        
        
        self._sep_expr=2*self.L.subs({self.w.diff(self.time):0,(self.w.diff(self.loc,2)):1}) .doit()  *Symbol('k',positive=True)**4 
        
#         print(self._sep_expr)

    def symbols_description(self):
        self.sym_desc_dict = {
            self.E: r'Young modulus',
            self.E: r'Young modulus',
            self.I: r'Cross-section moment of inertia',
            self.A: r'Area of cross-section',
            self.rho: r'density',
        }
        return self.sym_desc_dict
    
    def _bc_conditions(self):
            
        l=self.l
        x=self.loc
        X=Function('X')(x)
        
        
        sup_ls=Subs(X.diff(x,0),x,0)
        sup_rs=Subs(X.diff(x,0),x,l)

        fix_ls=Subs(X.diff(x,1),x,0)
        fix_rs=Subs(X.diff(x,1),x,l)

        mb_ls=Subs(X.diff(x,2),x,0)
        mb_rs=Subs(X.diff(x,2),x,l)

        v_ls=Subs(X.diff(x,3),x,0)
        v_rs=Subs(X.diff(x,3),x,l)
        
        return [sup_ls, sup_rs, fix_ls, fix_rs, mb_ls, mb_rs, v_ls, v_rs]

    def get_default_data(self):

        E_0, A_0, I_0, L_0 = symbols('E_0, A_0, I_0, L_0', positive=True)
        
        
        l=self.l
        x=self.loc
        X=Function('X')(x)
        
        
        sup_ls=Subs(X.diff(x,0),x,0)
        sup_rs=Subs(X.diff(x,0),x,l)

        fix_ls=Subs(X.diff(x,1),x,0)
        fix_rs=Subs(X.diff(x,1),x,l)

        mb_ls=Subs(X.diff(x,2),x,0)
        mb_rs=Subs(X.diff(x,2),x,l)

        v_ls=Subs(X.diff(x,3),x,0)
        v_rs=Subs(X.diff(x,3),x,l)
        
        

        default_data_dict = {
           self.E: [2.5*E_0,1.25*E_0,0.75*E_0,1.35*E_0,1*E_0,2*E_0,3*E_0,4*E_0,5*E_0,6*E_0,7*E_0,8*E_0,9*E_0 ],
           self.A: [1 * A_0, 2 * A_0, S.Half * A_0, 1 * A_0, 2 * A_0,3*A_0,4*A_0,5*A_0,6*A_0,7*A_0,8*A_0,9*A_0,10*A_0,11*A_0],
           self.I: [1 * I_0, 2 * I_0, S.Half * I_0, 1 * I_0, 2 * I_0,3*I_0,4*I_0,5*I_0,6*I_0,7*I_0,8*I_0,9*I_0,10*I_0,11*I_0],
           self.BC: [{sup_ls:0,mb_ls:0,sup_rs:0,mb_rs:0},{fix_ls:0,v_ls:0,sup_rs:0,mb_rs:0},{sup_ls:0,mb_ls:0,sup_rs:0,mb_rs:0},{fix_ls:0,v_ls:0,sup_rs:0,mb_rs:0},{sup_ls:0,mb_ls:0,sup_rs:0,mb_rs:0}],
           self.l:[0.8*L_0,1.2*L_0,1.4*L_0,2 * L_0, S.Half * L_0, 3 * L_0,4*L_0,5*L_0,6*L_0,7*L_0,8*L_0,9*L_0,10*L_0,11*L_0],

        }

        return default_data_dict

    def get_random_parameters(self):

        E_0, A_0, I_0, L_0 = symbols('E_0, A_0, I_0, L_0', positive=True)

        data_dict=super().get_random_parameters()
        data_dict[self.A]  = (L_0**2 * random.choice([0.125, 1.25, 0.0125, 1.23, 0.128 ])/ (data_dict[self.E]/E_0) / (data_dict[self.I]/I_0) ).n(3)

        return data_dict
    
    def bending_moment(self,
                   mode_no,
                   bc_dict=None,
                   sep_expr=None,
                   arg=None,
                   spatial_comp=Function('X')(Symbol('x')),
                   index=Symbol('n', integer=True, positive=True)):
        E_0, I_0, L_0 = symbols('E_0, I_0, L_0', positive=True)
        amp = Symbol('w_0',positive=True)
        
        mode = self.eigenmodes(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)

        return abs(mode.diff(self.loc,2)*Symbol('E',positive=True)*Symbol('I',positive=True))*amp

    def mode_cross_section_side(self,
                                mode_no,
                                bc_dict=None,
                                sep_expr=None,
                                arg=None,
                                spatial_comp=Function('X')(Symbol('x')),
                                index=Symbol('n', integer=True, positive=True)):
        
        Re,k_b,b,h = symbols('R_e k_b b h', positive=True)
        
        bending_moment =self.bending_moment(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)
        
#         h = sqrt(6*bending_moment/Re/k_a/b)
#         b = 6*bending_moment/Re/k_a/h**2
#         return b*h
        return (6*bending_moment/Re/k_b)**(1/3)

    def mode_moment_of_inertia(self,
                                mode_no,
                                bc_dict=None,
                                sep_expr=None,
                                arg=None,
                                spatial_comp=Function('X')(Symbol('x')),
                                index=Symbol('n', integer=True, positive=True)):
        
        Re,k_b,b,h = symbols('Re k_b b h', positive=True)
        
        bending_moment =self.bending_moment(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)
        
        return (bending_moment/Re/k_b)
    
    
class CSRod(ContinuousSystem):
    """
    Model of a Continuous Rod

        Arguments:
        =========
            E = Young modulus
                -Young modulus of a rod's material
            A = Area of cross-section

            rho_L = Linear density of a rod's material

            u = Space-Time function
                -Allow to describe a rod vibration in two-variable state

        Example
        =======
        A vibrating rod for given Young modulus and area of cross-section can be created in the following way.

        >>> t = symbols('t')
        >>> E,A,rho_L = symbols('E,A,rho_L',positive=True)
        >>> CSRod()

        -define the symbols and dynamicsymbols
        -determine the instance of the beam by using class CSRod()
    """

    scheme_name = 'rod_scheme.PNG'
    real_name = 'rod_real.PNG'

    @classmethod
    def from_random_data(cls):
        
        system=cls()
        subs_dict=system.get_random_parameters()
        
        return system.subs(subs_dict)
    
    def __init__(self,
                E=Symbol('E',positive=True),
                A=Symbol('A',positive=True),
                rho_L=Symbol('\\rho_L',positive=True),
                u=Function('u'),
                bc_dict=None,
                time=Symbol('t'),
                loc=Symbol('x'),
                **kwargs
                ):

        self.E = E
        self.A = A
        self.rho_L = rho_L
        self.u=u(time,loc)
        self.time=time
        self.loc=loc
        self.l=Symbol('L',positive=True)
        
        

        L_rod=S.One/2*(rho_L*(self.u.diff(self.time))**2-A*E*(self.u.diff(self.loc))**2)

        super().__init__(L_rod,q=self.u,bc_dict=bc_dict,t_var=self.time, spatial_var=self.loc,**kwargs)

        self._sep_expr=2*self.L.subs({self.u.diff(self.time):0,(self.u.diff(self.loc)):1}) .doit()  *Symbol('k',positive=True)**2 
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.E: r'Young modulus',
            self.A: r'Area of cross-section',
            self.rho_L: r'Linear density',
            self.u: 'Space-Time function',
        }
        return self.sym_desc_dict

    def _bc_conditions(self):
            
        x=self.loc
        l=self.l
        X=Function('X')(x)

        fix_ls=Subs(X.diff(x,0),x,0)
        fix_rs=Subs(X.diff(x,0),x,l)
        free_ls=Subs(X.diff(x,1),x,0)
        free_rs=Subs(X.diff(x,1),x,l)
        
        return [free_ls, free_rs, fix_ls, fix_rs]
    

    def get_default_data(self):

        E_0, A_0, L_0 = symbols('E_0, A_0, L_0', positive=True)
        
        x=self.loc
        l=self.l
        X=Function('X')(x)
        
        fix_ls=Subs(X.diff(x,0),x,0)
        fix_rs=Subs(X.diff(x,0),x,l)
        free_ls=Subs(X.diff(x,1),x,0)
        free_rs=Subs(X.diff(x,1),x,l)


        default_data_dict = {
           self.E: [2.5*E_0,1.25*E_0,0.75*E_0,1.35*E_0 ],
           self.A: [1 * A_0, 2 * A_0, S.Half * A_0, 1 * A_0, 2 * A_0],
           self.BC: [{fix_ls:0,fix_rs:0},{free_ls:0,fix_rs:0},{fix_ls:0,free_rs:0},{free_ls:0,free_rs:0},{fix_ls:0,free_rs:0}],
           self.l:[1 * L_0, 2 * L_0, S.Half * L_0, 1 * L_0, 2 * L_0],
            }

        return default_data_dict

    def get_random_parameters(self):

        E_0, A_0, L_0 = symbols('E_0, A_0, L_0', positive=True)

        data_dict=super().get_random_parameters()
#         data_dict[self.A]  = (L_0**2 * random.choice([0.125, 0.0125, 0.00125, 0.123, 0.0128 ])/ (data_dict[self.E]/E_0) ).n(3)
        data_dict[self.A]  = (L_0**2 * random.uniform(0.00125, 0.125)/ (data_dict[self.E]/E_0) ).n(3)
        return data_dict

    
    def normal_force(self,
                   mode_no,
                   bc_dict=None,
                   sep_expr=None,
                   arg=None,
                   spatial_comp=Function('X')(Symbol('x')),
                   index=Symbol('n', integer=True, positive=True)):
        E_0, A_0, L_0 = symbols('E_0, A_0, L_0', positive=True)
        amp = Symbol('u_0',positive=True)
        
        mode = self.eigenmodes(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)
#         display(self._given_data)
        
#         display(mode)
        
        
        
        return (mode.diff(self.loc)*Symbol('E',positive=True)*Symbol('A',positive=True))*amp
    


    
    def determine_section_side_for_mode(self,
                   mode_no,
                   bc_dict=None,
                   sep_expr=None,
                   arg=None,
                   spatial_comp=Function('X')(Symbol('x')),
                   index=Symbol('n', integer=True, positive=True)):
        
        sigma_n = symbols('sigma_n', positive=True)
        Re,k_n,b,h = symbols('R_e k_n b h', positive=True)
        
        force =self.normal_force(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)
        
        return sqrt(Abs(force)/(Re*k_n))
    
    
class CSString(ContinuousSystem):


    scheme_name = 'string_scheme.png'
    real_name = 'double_bass_string.jpg'

    def __init__(self,
                Ty=Symbol('T_y',positive=True),
                A=Symbol('A',positive=True),
                rho=Symbol('\\rho',positive=True),
                w=Function('w'),
                bc_dict=None,
                time=Symbol('t'),
                loc=Symbol('x'),
                **kwargs
                ):

        self.Ty = Ty
        self.A = A
        self.rho = rho
        self.w=w(time,loc)
        self.time=time
        self.loc=loc
        self.l=Symbol('L',positive=True)
        
        

        L_rod=S.One/2*(A*rho*(self.w.diff(self.time))**2-Ty*(self.w.diff(self.loc))**2)

        super().__init__(L_rod,q=self.w,bc_dict=bc_dict,t_var=self.time, spatial_var=self.loc,**kwargs)

        self._sep_expr=2*self.L.subs({self.w.diff(self.time):0,(self.w.diff(self.loc)):1}) .doit()  *Symbol('k',positive=True)**2 

        
    @classmethod
    def from_random_data(cls):
        
        system=cls()
        subs_dict=system.get_random_parameters()
        
        return system.subs(subs_dict)
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.Ty: r'Initial tension',
            self.A: r'Area of cross-section',
            self.rho: r'Material density',
            self.w: 'Space-Time function',
        }
        return self.sym_desc_dict
    
    def _bc_conditions(self):
            
        x=self.loc
        l=self.l
        X=Function('X')(x)

        fix_ls=Subs(X.diff(x,0),x,0)
        fix_rs=Subs(X.diff(x,0),x,l)
        free_ls=Subs(X.diff(x,1),x,0)
        free_rs=Subs(X.diff(x,1),x,l)
        
        return [free_ls, free_rs, fix_ls, fix_rs]



    def get_default_data(self):

        T0, A_0, L_0 = symbols('T_0, A_0, L_0', positive=True)
        
        x=self.loc
        l=self.l
        X=Function('X')(x)
        
        fix_ls=Subs(X.diff(x,0),x,0)
        fix_rs=Subs(X.diff(x,0),x,l)
        free_ls=Subs(X.diff(x,1),x,0)
        free_rs=Subs(X.diff(x,1),x,l)


        default_data_dict = {
            self.Ty:[T0 * random.uniform(0.75, 2.5)],
            
            self.A: [1 * A_0, 2 * A_0, S.Half * A_0, 1 * A_0, 2 * A_0],
            
           self.BC: [{fix_ls:0,fix_rs:0} ],
           self.l:[1 * L_0, 2 * L_0, S.Half * L_0, 3 * L_0, 2 * L_0,4*L_0,5*L_0,6*L_0,7*L_0,8*L_0,9*L_0],

        }

        return default_data_dict

    def get_random_parameters(self):
        
        T0, A_0, L_0 = symbols('T_0, A_0, L_0', positive=True)
        
        data_dict=super().get_random_parameters()
        data_dict[self.A]  = (L_0**2 * random.choice([0.03, 0.031, 0.0031, 0.033, 3.1 ])/ (data_dict[self.Ty]/T0) ).n(3)
        
        
        return data_dict

class CSShaft(ContinuousSystem):


    scheme_name = 'supported_shaft_scheme.PNG'
    real_name = 'shaft_real.PNG'

    def __init__(self,
                G=Symbol('G',positive=True),

                M=Symbol('M',positive=True),
                I=Symbol('I',positive=True),

                rho=Symbol('\\rho',positive=True),
                phi=Function('\\phi'),
                bc_dict=None,
                time=Symbol('t'),
                loc=Symbol('x'),
                **kwargs
                ):


        self.M = M
        self.G = G
        self.I = I

        self.rho = rho
        self.phi=phi(time,loc)
        self.time=time
        self.loc=loc
        self.l=Symbol('L',positive=True)
        
        


        L_shaft=S.One/2*(rho*I*(self.phi.diff(self.time))**2-G*I*(self.phi.diff(self.loc))**2)


        super().__init__(L_shaft,q=self.phi,bc_dict=bc_dict,t_var=self.time, spatial_var=self.loc,**kwargs)

        self._sep_expr=2*self.L.subs({self.phi.diff(self.time):0,(self.phi.diff(self.loc)):1}) .doit()  *Symbol('k',positive=True)**2 

        
    @classmethod
    def from_random_data(cls):
        
        system=cls()
        subs_dict=system.get_random_parameters()
        
        return system.subs(subs_dict)
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.G: r'''Kirchoff's modulus ''',
            
            self.rho: r'Material density',
            self.phi: 'Space-Time function',
        }
        return self.sym_desc_dict
    
    def _bc_conditions(self):
            
        x=self.loc
        l=self.l
        X=Function('X')(x)

        fix_ls=Subs(X.diff(x,0),x,0)
        fix_rs=Subs(X.diff(x,0),x,l)
        free_ls=Subs(X.diff(x,1),x,0)
        free_rs=Subs(X.diff(x,1),x,l)
        
        return [free_ls, free_rs, fix_ls, fix_rs]

    def get_default_data(self):

        G_0, L_0 = symbols('G_0, L_0', positive=True)
        
        x=self.loc
        l=self.l
        X=Function('X')(x)
        
        fix_ls=Subs(X.diff(x,0),x,0)
        fix_rs=Subs(X.diff(x,0),x,l)
        free_ls=Subs(X.diff(x,1),x,0)
        free_rs=Subs(X.diff(x,1),x,l)


        default_data_dict = {
           self.G: [2.5*G_0,1.25*G_0,0.75*G_0,1.35*G_0,1.5*G_0,1.45*G_0],
           self.I: [L_0**4/120, L_0**4/20,L_0**4/600,L_0**4/100,L_0**4/2],
           self.BC: [{fix_ls:0,fix_rs:0},{free_ls:0,fix_rs:0},{fix_ls:0,free_rs:0},{free_ls:0,free_rs:0},{fix_ls:0,free_rs:0}],
           self.l:[L_0 *(no+5)*S.One*10 for no in range(1,8)],


        }

        return default_data_dict

    def get_random_parameters(self):
        
        G_0, L_0 = symbols('G_0, L_0', positive=True)
        
        data_dict=super().get_random_parameters()

        data_dict[self.I]  = (L_0**4 * random.choice([ 1.1,11,110,10,111 ])/ (data_dict[self.G]/G_0) ).n(3)

        
        
        return data_dict
    
    def twisting_moment(self,
                   mode_no,
                   bc_dict=None,
                   sep_expr=None,
                   arg=None,
                   spatial_comp=Function('X')(Symbol('x')),
                   index=Symbol('n', integer=True, positive=True)):
        G_0, I_0, L_0 = symbols('G_0, I_0, L_0', positive=True)
        amp = Symbol('\\phi_0',positive=True)
        
        mode = self.eigenmodes(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)

        return abs(mode.diff(self.loc)*Symbol('G',positive=True)*Symbol('I',positive=True))*amp
    
    def mode_cross_section_diameter(self,
                                mode_no,
                                bc_dict=None,
                                sep_expr=None,
                                arg=None,
                                spatial_comp=Function('X')(Symbol('x')),
                                index=Symbol('n', integer=True, positive=True)):
        
        Re,k_t,d = symbols('R_e k_t d', positive=True)
        
        twisting_moment = self.twisting_moment(mode_no,
                   bc_dict=bc_dict,
                   sep_expr=sep_expr,
                   arg=arg,
                   spatial_comp=spatial_comp,
                   index=index)
        
        return (16*twisting_moment/pi/Re/k_t)**(1/3)


class CSCylinder(PlaneStressProblem):
    
    scheme_name = 'cylinder_scheme.PNG'
    real_name = 'cylinder_real_cannon.PNG'
    
    def __init__(self,
                 
                 disp_func=[Function('\\mathit{u}')(Symbol('r')), 0],
                 stress_tensor=Matrix(2, 2, [
                     Function('\\sigma_r')(Symbol('r')), 0, 0,
                     Function('\\sigma_\\varphi')(Symbol('r'))
                 ]),

                 bc_dict=None,
                 coords=[Symbol('r'), Symbol('\\varphi')],
                 E=Symbol('E', positive=True),
                 nu=Symbol('\\nu', positive=True),
                 D=Symbol('D', positive=True),
                 volumetric_load=0,
                 **kwargs
                ):
        
        self.disp_func = disp_func
        self.coords = coords
        self.ra = Symbol('r_a',positive=True)
        self.rb = Symbol('r_b',positive=True)
        

        
        super().__init__(disp_func=disp_func,stress_tensor=stress_tensor,bc_dict=bc_dict,coords=coords,E=E,nu=nu,D=D,volumetric_load=volumetric_load,**kwargs)
        
    def get_default_data(self):

        r_0, p_0 = symbols('r_0, p_0', positive=True)
        p0=p_0
        
        func_r=self.disp_func[0]
        r=self.coords[0]
        nu=self.poisson
        E=self.E_module
        D=self.D
        ra=self.ra
        rb=self.rb
        
        in_p0=Subs((func_r.diff(r)+nu*func_r/r)*D,r,ra)
        out_p0=Subs((func_r.diff(r)+nu*func_r/r)*D,r,rb)

        default_data_dict = {
           self.ra: [S.One*r_0,2.05*r_0,0.5*r_0,0.75*r_0,1.5*r_0,0.75*r_0],
           self.rb: [S.One*r_0,3.15*r_0,1.5*r_0,0.85*r_0,1.35*r_0,0.55*r_0],
           self.BC: [{in_p0:p0,out_p0:S.Zero},{in_p0:S.Zero,out_p0:p0},{in_p0:p0,out_p0:p0},{in_p0:-p0,out_p0:S.Zero},{in_p0:S.Zero,out_p0:-p0},{in_p0:-p0,out_p0:-p0}],
        }

        
        return default_data_dict

    
    
class CSPlate(PlaneStressProblem):
    
#     scheme_name = 'plate_point_load.PNG'
    scheme_name = 'plate_scheme.PNG'
    real_name = 'reservoir.jpg'
    
    def __init__(self,
                 disp_func=[Function('\\psi')(Symbol('r')), 0],
                 stress_tensor=Matrix(2, 2, [
                     Function('\\mathit{m}_r')(Symbol('r')), 0, 0,
                     Function('\\mathit{m}_\\varphi')(Symbol('r'))
                 ]),
                 bc_dict=None,
                 coords=[Symbol('r'), Symbol('\\varphi')],
                 E=Symbol('E', positive=True),
                 nu=Symbol('\\nu', positive=True),
                 D=Symbol('D_h', positive=True),
                 h=Symbol('h', positive=True),
                 volumetric_load=0,
                 **kwargs
                ):

        self.disp_func = disp_func
        self.coords = coords
        self.ra = Symbol('r_a',positive=True)
        self.rb = Symbol('r_b',positive=True)
        
        super().__init__(disp_func=disp_func,stress_tensor=stress_tensor,bc_dict=bc_dict,coords=coords,E=E,nu=nu,D=D,volumetric_load=volumetric_load,**kwargs)

        self._h=h
        
        self._subs_dict = {E:D*(1-nu**2)*12/(h**3)} 
        self.E_module=(h**3)/12*E
#         self.u = Matrix([Function('\\psi')(Symbol('r')), 0])

    def get_default_data(self):

        r_0, m_0 = symbols('r_0, m_0', positive=True)
        m0=m_0
        
        func_r=self.disp_func[0]
        r=self.coords[0]
        nu=self.poisson
        E=self.E_module
        D=self.D
        ra=self.ra
        rb=self.rb
        
        in_m0=Subs((func_r.diff(r)+nu*func_r/r)*D,r,ra)
        out_m0=Subs((func_r.diff(r)+nu*func_r/r)*D,r,rb)

        default_data_dict = {
           self.ra: [S.One*r_0,2.4*r_0,0.6*r_0,0.8*r_0,1.6*r_0,0.9*r_0],
           self.rb: [S.One*r_0,3.0*r_0,1.*r_0,0.2*r_0,1.4*r_0,0.6*r_0],
           self.BC: [{in_m0:m0,out_m0:S.Zero},{in_m0:S.Zero,out_m0:m0},{in_m0:m0,out_m0:m0},{in_m0:-m0,out_m0:S.Zero},{in_m0:S.Zero,out_m0:-m0},{in_m0:-m0,out_m0:-m0}],
        }

        
        return default_data_dict


from sympy import (Symbol, symbols, Matrix, sin, cos, diff, sqrt, S, diag, Eq,
                   hessian, Function, flatten, Tuple, im, re, pi, latex,
                   dsolve, solve, fraction, factorial, Add, Mul, exp, zeros, shape,
                   numbered_symbols, integrate, ImmutableMatrix,Expr,Dict,Subs,Derivative,Dummy,
                   lambdify, Pow, Integral, init_printing)

from sympy.matrices.matrices import MatrixBase
from numbers import Number

###  exemplary comment
from sympy.physics.mechanics import dynamicsymbols, init_vprinting
from sympy.physics.vector.printing import vpprint, vlatex
from ..utilities.components.ode import en as ode

import pandas as pd
from ..utilities.adaptable import *
import sympy as sym
from sympy.printing import *
import pint
import inspect
from dynpy.utilities.documents import document
from dynpy.utilities.templates.document import Guide
from dynpy.utilities.report import Markdown


ODE_COMPONENTS_LIST = [
            ode.ODESystemComponent,
            ode.ODEInitComponent,
            ode.ODESystemCodeComponent,
            ode.VariablesComponent,
            ode.HomEquationComponent,
            ode.HomEquationCodeComponent,
            ode.GoverningEquationComponent,
            ode.FundamentalMatrixComponent,
#             ode.ODECharecteristicPolynomialComponent,
            ode.ODECharecteristicPolynomialCodeComponent,
            ode.GeneralSolutionComponent
]

class CommonFactorDetector:
    
    _default_symbol = Symbol('\\tau^2',positive=True)
    
    def __init__(self,fundamental_matrix,default_symbol=None):
        self._fundametnal_matrix  = fundamental_matrix
        
        if default_symbol is not None: self._default_symbol = default_symbol
            
        
    def _const_spotter(self):
        
        
        nums_list = [list(fraction(elem)[0].atoms(Symbol)) for elem in self._fundametnal_matrix]
        denoms_list = [list(fraction(elem)[1].atoms(Symbol)) for elem in self._fundametnal_matrix]
       
        num_common=set(sum(nums_list,[]))
        for elem in nums_list:
            #print(elem,elem != set())
            if set(elem) != set():
                
                num_common &= set(elem)

                
        denom_common=set(sum(denoms_list,[]))
        for elem in denoms_list:
            #print(elem,elem != set())
            if set(elem) != set():
                
                denom_common &= set(elem)
        
        
        
        
        return (list(num_common)+[None])[0],(list(denom_common)+[None])[0]


    def subs_dict(self):
        
        num, denom  = self._const_spotter()
        
        
        
        if num is not None and denom is not None:
            return {num: denom / self._default_symbol}
        else:
            return {}
    
    def callback_dict(self):
        
        num, denom  = self._const_spotter()
        
        if num is not None and denom is not None:
            return {self._default_symbol: denom / num}
        else:
            return {}
        
        
class CodeFlowLogger:
    _display = False
    # _obj_printer = lambda obj: (print(f'str: ',obj,'preview'),display(obj),print(f'type of arg: {type(obj)}'))
    # _cls_printer = lambda obj: (print(f'str: ',obj,'preview'),print(f'type of arg: {type(obj)}'))
    
    def __init__(self,entry=None,comment = None,environment = None ):
        
        self._entry = entry
        self._env = environment
        self._comment = comment
        
        if self.__class__._display:
            self._report_status()
            
        
    def _report_status(self):
    
        print(f'START !!!!!!!!!!!!!! {self._comment} !!!!!!!!!!!!!!')    
        if self._entry is not None:
            print('#### Entry:')
            self._entry_report()
            
        if self._env is not None:
            print('#### Env')
            self._env_report()
            
        print(f'END !!!!!!!!!!!!!! {self._comment} !!!!!!!!!!!!!!')     
    
    def _entry_report(self):
        
        obj = self._entry
        
        print(f'str:',obj)
        print('preview with display:')
        display(obj)
        print(f'type of arg: {type(obj)}')        

    def _env_report(self):
        
        obj = self._env
        
        print(f'str: ',obj)
        print(f'type of arg: {type(obj)}')   
        
        
        
        
class TwoSystemsSimulation:

    def __init__(self,odes, t_span=None):
        self._odes = odes
        self._t_span = t_span
        
        
    @property
    def odes(self):

        odes = self._odes

        if not isinstance(odes,tuple):
            odes = (odes,odes)

        return odes

    def numerized(self):
        return self
    
    def compute_solution(self,t_span,ic_list=[0.0,0.0,0.0,0.0]):

        from .linear import FirstOrderLinearODESystemWithHarmonics
        
        ode1 , ode2 = self.odes

        t_span0 = t_span[0]
        t_span1 = t_span[1]

        fode_1=ode1.as_first_ode_linear_system()
        fodes_1=FirstOrderLinearODESystemWithHarmonics(fode_1,fode_1.dvars)
#         ode2.as_first_ode_linear_system()
#         fodes_2=FirstOrderLinearODESystemWithHarmonics(ode2,ode2.dvars)

        fode_2=ode2.as_first_ode_linear_system()
        fodes_2=FirstOrderLinearODESystemWithHarmonics(fode_2,fode_2.dvars)

        ana_sol_sym_1=fodes_1.solution.with_ics(ic_list)
        ana_sym_wyn_1=pd.DataFrame(ana_sol_sym_1.numerized().compute_solution(t_span=t_span0))

        ana_sol_sym_2=fodes_2.solution.with_ics(sol0=ana_sol_sym_1, ivar0=t_span1[0])
        ana_sym_wyn_2=pd.DataFrame(ana_sol_sym_2.numerized().compute_solution(t_span=t_span1))

        return TimeDataFrame(pd.concat([ana_sym_wyn_1,ana_sym_wyn_2]))
        
        
class CodePrinter:
    
    def __init__(self, expr, *args, **kwargs):
        self._expr=expr
        
        
        
    def _generate_code(self):

        expr=self._expr
        raw_code = python(expr).replace("e = ","eq = ")

        for elem in expr.atoms():
            if isinstance(elem, (sym.Float, sym.Integer)):
                pass
            elif isinstance(elem, (sym.Symbol)):
                my_arg = str(elem)
                raw_code = raw_code.replace(f"Symbol('{my_arg}')",srepr(elem))
            else:
                pass
            
         
        return raw_code
        
    def _print_code(self):

        return print(self._generate_code())

class LatexSize:

    _units= pint.UnitRegistry()
    _units.default_format = "~"

    def __init__(self, size, unit=None,):
        self._size=size
        self._unit=unit
    @classmethod
    def units(cls):
        return cls._units

    def _size_with_unit(self):
        ureg = self.units()
        return ureg(self._size)
    @property
    def size(self):
        return self._size_with_unit().to(ureg.cm)

    @property
    def unit(self):
        ureg = self.units()
        if self._unit is None:
            return ureg('dm')
        else:
            return ureg(self._unit)





"""
This module provides basic tools for solving differential equations
"""


import importlib

# import .dynamics as dyn
# importlib.reload(dyn)

# import .utilities.timesiries as ts
# importlib.reload(ts)


import dynpy
from dynpy.utilities.report import ReportText
from .report import Markdown
from .documents.document import ODESystemOverviewReport
from github import Github
import getpass

import pint
import inspect
from dynpy.utilities.documents import document
from dynpy.utilities.templates.document import Guide
from datetime import datetime

issue_no = 567
title = 'implementation of overview report for `ODESystem` child classes'
guide_class = ODESystemOverviewReport

meet_issue_code_str = """The following actions are to do, in order to prepare the meeting and the guide:

- [{tic}] #{issue_no+1}

- [{tic}] #{issue_no+2}

- [{tic}] #{issue_no+3}

"""


meet_time_issue_code_str = """The goal is to schedule the meeting in Teams calendar on {date} at {time}.

"""

meet_arrange_issue_code_str ="""The goal is to arrange and execute the meeting for {title}. The following things should be prepared:

- [{tic}] scope and schedule of meeting (in one week);

- [{tic}] conducting team (in two weeks);

- [{tic}] guide or instructions with examples (one week before meeting);

- [{tic}] announcement (in Slack and Teams channel) with basic information (conducting persons, date,scope,schedule) (one week before meeting);

- [{tic}] exemplary code in Jupyter (link) (one week before meeting).

"""


meet_guide_issue_code_str ="""The goal is to prepare or improve a guide or report that is generated by the following code:

```python
from {guide_class_module} import {guide_class_name}
{guide_class_name}()
```

"""

class MeetingIssueCreator:

    """

    Example:
    from dynpy.utilities.creators import MeetingIssueCreator
    from dynpy.utilities.documents.document import IntroDynPyProjectGuidelines
    title="Issue"
    last_issue_no=533
    MeetingIssueCreator(title=title,no=last_issue_no+1,guide=IntroDynPyProjectGuidelines,date='2024.09.26',time='20:30',done=False)

    """

    _title = 'implementation of ODESystem overview report'
    _issue_no = 567
    _guide = ODESystemOverviewReport
    _time = '20:30'
    _date = '2024.07.27'
    _assignees = ['bogumilchilinski']

    def __init__(self,title=None,no=None,guide=None,date=None,time=None,assignees=None,done=False,*args,**kwargs):
        if title is not None: self._title = title
        if no is not None: self._issue_no = no
        if guide is not None: self._guide = guide
        if date is not None: self._date= date
        if time is not None: self._time = time
        self._done = done
        if assignees is not None: self._assignees = assignees

    def _get_elems_dict(self):

        if self._done: tic = 'x'
        else: tic = ' '

        guide_class = self._guide

        elems_dict = {
                'title':self._title,
                'issue_no':self._issue_no,
                'issue_no+1':self._issue_no+1,
                'issue_no+2':self._issue_no+2,
                'issue_no+3':self._issue_no+3,
                'guide_class_module':guide_class.__module__,
                'guide_class_name':guide_class.__name__,
                'date':self._date,
                'time':self._time,
                'tic':tic
                }

        return elems_dict

    def get_issues_titles(self):

        # Issue no. #{issue_no}
        ## Preparation of meeting and guide for {title}

        issue_no = self._issue_no
        title = self._title

        titles_dict =   {
                        'meet':f'Preparation of meeting and guide for {title}',
                        'time':f'Scope and time scheduling of the meeting for {title} (issue #{issue_no} related)',
                        'arrange':f'Arrangement and execution of the meeting for {title} (issue #{issue_no} related)',
                        'guide':f'Preparation or improvement of the guide for {title} (issue #{issue_no} related)',
                        }

        return titles_dict

    def get_issue_str(self):
        elems_dict = self._get_elems_dict()
        issue_no = self._issue_no

        titles = self.get_issues_titles()

        meet_issue_dict = self.get_meet_issue_dict()
        time_issue_dict = self.get_time_issue_dict()
        arrange_issue_dict = self.get_arrange_issue_dict()
        guide_issue_dict = self.get_guide_issue_dict()


        issue_code_str = (f'# Issue no. #{issue_no} \n '+
                        '## '+meet_issue_dict['title'] + '\n ' +
                        meet_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+1} \n '+
                        '## '+ time_issue_dict['title'] + '\n ' +
                        time_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+2} \n '+
                        '## '+arrange_issue_dict['title'] + '\n ' +
                        arrange_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+3} \n ' +
                        '## '+guide_issue_dict['title'] + '\n ' +
                        guide_issue_dict['body'])

        return issue_code_str


    def get_meet_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()

        return {'title':titles['meet'],'body':meet_issue_code_str.format(**elems_dict)}

    def get_time_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()

        return {'title':titles['time'],'body':meet_time_issue_code_str.format(**elems_dict)}


    def get_arrange_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()

        return {'title':titles['arrange'],'body':meet_arrange_issue_code_str.format(**elems_dict)}

    def get_guide_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()

        return {'title':titles['guide'],'body':meet_guide_issue_code_str.format(**elems_dict)}

    def _repr_markdown_(self):
        return self.get_issue_str()

    @property
    def closing_comment_text_meet(self):
        
        # obj_class_module= self._obj.__module__
        # obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'All done.'

        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}'


    @property
    def closing_comment_text_time(self):
        
        # obj_class_module= self._obj.__module__
        # obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'Meeting has been scheduled.'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}'

    @property
    def closing_comment_text_arrange(self):
        
        # obj_class_module= self._obj.__module__
        # obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'Meeting has been executed.'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}'

    @property
    def closing_comment_text_guide(self):
        
        
        obj_class_module= self._guide.__module__
        obj_class_name=self._guide.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'Minor changes introduced. The code was checked with the following call:'
        comment_call_code = f'```python \nfrom {obj_class_module} import {obj_class_name} \n{obj_class_name}(None) \n```'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}\n{comment_call_code}'


    def publish_on_github(self,repository):

        guide_label=repository.get_label('guidance')
        assignees=self._assignees

        issue_meet=repository.create_issue(**self.get_meet_issue_dict(),labels=[guide_label],assignees=assignees
                                )
        issue_time=repository.create_issue(**self.get_time_issue_dict(),labels=[guide_label],assignees=['bogumilchilinski'])
        issue_arrange=repository.create_issue(**self.get_arrange_issue_dict(),labels=[guide_label],assignees=assignees
                                )
        issue_guide=repository.create_issue(**self.get_guide_issue_dict(),labels=[guide_label],assignees=assignees
                                )

        issue_guide.create_comment(self.closing_comment_text_guide)
        issue_arrange.create_comment(self.closing_comment_text_arrange)
        issue_time.create_comment(self.closing_comment_text_time)
        issue_meet.create_comment(self.closing_comment_text_meet)

        return issue_meet,issue_time,issue_arrange,issue_guide





class HelpImplementationIssueCreator:

    """
    Example:
    from dynpy.utilities.creators import MeetingIssueCreator
    from dynpy.utilities.documents.document import IntroDynPyProjectGuidelines
    title="Issue"
    last_issue_no=533
    MeetingIssueCreator(title=title,no=last_issue_no+1,guide=IntroDynPyProjectGuidelines,date='2024.09.26',time='20:30',done=False)

    """
    
    _obj = ODESystemOverviewReport
    _issue_no = 567
    _guide = ODESystemOverviewReport
    _time = '20:30'
    _date = '2024.07.27'
    _assignees = ['bogumilchilinski']
    _module = 'exemplary.module'
    _default_labels = ['documentation','enhancement','maintenance']
    _issue_type = 'Help'
    _goal = 'creates a part of a report'

    def __init__(self,obj=None,goal=None,module=None,no=None,assignees=None,done=False,*args,**kwargs):
        if obj is not None: self._obj = obj
        if no is not None: self._issue_no = no
        if module is not None: self._module = module
        if goal is not None: self._goal = goal

        self._done = done
        if assignees is not None: self._assignees = assignees

    def _get_elems_dict(self):

        if self._done: tic = 'x'
        else: tic = ' '

        obj = self._obj

        elems_dict = {
                'obj':obj,
                'goal':self._goal,
                'issue_no':self._issue_no,
                'issue_no+1':self._issue_no+1,
                'issue_no+2':self._issue_no+2,
                'issue_no+3':self._issue_no+3,
                'obj_class_module':obj.__module__,
                'obj_class_name':obj.__name__,
                # 'date':self._date,
                # 'time':self._time,
                'tic':tic
                }

        return elems_dict

    @property
    def title(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        
        return f'Help for `{obj_class_name}` class from `{obj_class_module}` module'

    @property
    def body_text(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        
        issue_code_str=(f'Help *docstring* is to implement for `{obj_class_name}` class that is defined in `{obj_class_module}`. '+
        'It should list of all available information about class.')
        
        return issue_code_str

    @property
    def closing_comment_text(self):
        
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'Help *docstring* added. \n\nThe code was checked with the following call:'
        comment_call_code = f'```python \nfrom {obj_class_module} import {obj_class_name} \n{obj_class_name}() \n```'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}\n{comment_call_code}'



    def get_issues_titles(self):

        # Issue no. #{issue_no}
        ## Preparation of meeting and guide for {title}



        titles_dict =   {
                        'issue':self.title,
                        }

        return titles_dict

    def get_issue_str(self):
        elems_dict = self._get_elems_dict()
        issue_no = self._issue_no

        titles = self.get_issues_titles()

        issue_dict = self.get_issue_dict()

        issue_code_str = (f'# Issue no. #{issue_no} \n '+
                        '## '+issue_dict['title'] + '\n ' +
                        issue_dict['body'] + '\n ' +
                        '### Closing comment (example)' + '\n ' +
                        self.closing_comment_text
                        )

        return issue_code_str



    def get_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()

        issue_code_str=('Help *docstring* is to implement for `{obj_class_name}` class that is defined in `{obj_class_module}`. '+
        'It should list of all available information about class.')

        return {'title':self.title,'body':self.body_text.format(**elems_dict)}


    def _repr_markdown_(self):
        return self.get_issue_str()


    def publish_on_github(self,repository,labels=[]):

        labels = self._default_labels+labels

        issue_labels=[repository.get_label(label_arg) for label_arg in labels]
        assignees=self._assignees

        issue_help=repository.create_issue(**self.get_issue_dict(),labels=issue_labels,assignees=assignees
                                )
        
        issue_help.create_comment(self.closing_comment_text)

        return issue_help


class ReportComponentImplementationIssueCreator(HelpImplementationIssueCreator):
    
    _default_labels = ['documentation','enhancement','guide','maintenance','reporting']
    _issue_type = 'Implementation'
    _goal = 'creates a part of a report'
    
    @property
    def title(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        goal = self._goal
        
        issue_type = self._issue_type
        
        return f'{issue_type} of `{obj_class_name}` class that {goal} (defined in `{obj_class_module}` module)'

    @property
    def body_text(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        goal = self._goal
        
        

        issue_code_str=(f'`{obj_class_name}` class (report component) is to implement in `{obj_class_module}` module. '+
        'It should provide code that {goal}.')
        
        return issue_code_str

    @property
    def closing_comment_text(self):
        
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'The code was checked with the following call:'
        comment_call_code = f'```python \nfrom {obj_class_module} import {obj_class_name} \n{obj_class_name}(None) \n```'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}\n{comment_call_code}'


class ReportComponentDevelopmentIssueCreator(HelpImplementationIssueCreator):
    
    _default_labels = ['documentation','enhancement','guide','maintenance','reporting']
    _issue_type = 'Development'
    _goal = 'creates a part of a report'
    
    @property
    def title(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        goal = self._goal
        
        issue_type = self._issue_type
        
        return f'{issue_type} of `{obj_class_name}` class that {goal} (defined in `{obj_class_module}` module)'

    @property
    def body_text(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        goal = self._goal
        
        

        issue_code_str=(f'`{obj_class_name}` class (report component) is to develop in `{obj_class_module}` module. '+
        'It should provide code that {goal}.')
        
        return issue_code_str

    @property
    def closing_comment_text(self):
        
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'The code was checked with the following call:'
        comment_call_code = f'```python \nfrom {obj_class_module} import {obj_class_name} \n{obj_class_name}(None) \n```'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}\n{comment_call_code}'


class SyncIssueCreator(ReportComponentDevelopmentIssueCreator):
    
    _default_labels = ['daily','maintenance','sync']
    _issue_type = 'Sync'
    _goal = 'creates a part of a report'
 
    def _get_elems_dict(self):

        if self._done: tic = 'x'
        else: tic = ' '

        obj = self._obj

        elems_dict = {
                'obj':obj,
                'goal':self._goal,
                'issue_no':self._issue_no,
                'issue_no+1':self._issue_no+1,
                'issue_no+2':self._issue_no+2,
                'issue_no+3':self._issue_no+3,
                # 'obj_class_module':obj.__module__,
                # 'obj_class_name':obj.__name__,
                # 'date':self._date,
                # 'time':self._time,
                'tic':tic
                }

        return elems_dict
 
    
    @property
    def title(self):
        
        # issue_no = self._issue_no
        # obj_class_module= self._obj.__module__
        # obj_class_name=self._obj.__name__
        # goal = self._goal
        date = self._obj
        
        issue_type = self._issue_type
        
        return f'{issue_type} of GitHub repositories within {date}'

    @property
    def body_text(self):
        
        # issue_no = self._issue_no
        # obj_class_module= self._obj.__module__
        # obj_class_name=self._obj.__name__
        # goal = self._goal
        
        

        issue_code_str=(f'The aim is to synchronize GitHub repositories in the given period.')
        
        return issue_code_str

    @property
    def closing_comment_text(self):
        
        # obj_class_module= self._obj.__module__
        # obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'Repositories are synchronised. Goal is achieved.'
        
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}'


class ClassImplementationIssueCreator(HelpImplementationIssueCreator):
    
    _default_labels = ['enhancement','module']
    _issue_type = 'Implementation'
    _goal = 'creates a part of a report'
    
    @property
    def title(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        goal = self._goal
        
        issue_type = self._issue_type
        
        return f'{issue_type} of `{obj_class_name}` class that {goal} (defined in `{obj_class_module}` module)'

    @property
    def body_text(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        goal = self._goal
        
        

        issue_code_str=(f'`{obj_class_name}` class is to implement in `{obj_class_module}` module. '+
        'It should provide code that {goal}.')
        
        return issue_code_str

    @property
    def closing_comment_text(self):
        
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__name__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'The code was checked with the following call:'
        comment_call_code = f'```python \nfrom {obj_class_module} import {obj_class_name} \n{obj_class_name}(None) \n```'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}\n{comment_call_code}'

class MethodImplementationIssueCreator(HelpImplementationIssueCreator):
    
    _default_labels = ['enhancement','module']
    _issue_type = 'Implementation'
    _goal = 'creates a part of a report'
    
    @property
    def title(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__qualname__
        goal = self._goal
        
        issue_type = self._issue_type
        
        return f'{issue_type} of `{obj_class_name}` method that {goal} (defined in `{obj_class_module}` module)'

    @property
    def body_text(self):
        
        issue_no = self._issue_no
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__qualname__
        goal = self._goal
        
        

        issue_code_str=(f'`{obj_class_name}` method is to implement in `{obj_class_module}` module. '+
        'It should provide code that {goal}.')
        
        return issue_code_str

    @property
    def closing_comment_text(self):
        
        obj_class_module= self._obj.__module__
        obj_class_name=self._obj.__qualname__
        
        comment_str_h1 = '> *Exemplary comment. You can copy example below and close issue with new comment or modify it and close the issue.* '
        comment_str_h2 = '*Remove this message or keep it if new comment is created.*'
        
        comment_str_2 = 'The code was checked with the following call:'
        comment_call_code = f'```python \nfrom {obj_class_module} import {obj_class_name} \n{obj_class_name}(None) \n```'
        
        return f'{comment_str_h1}{comment_str_h2}\n\n{comment_str_2}\n{comment_call_code}'



class GitHubInterface():

    '''
    GitHubInterface is a class to use the Github API v3. With it, you can manage your Github resources (repositories, user profiles, organizations, etc.) from Python scripts.
    
    It is based on Github class from PyGithub library and it mimics its methods.

    Examples:

    from dynpy.utilities.creators import GitHubInterface

    # Authentication is defined via github.Auth
    # using an access token to Public Web Github

    client = GitHubInterface()

    repos = client.get_issues_list(repo_name="bogumilchilinski/dynpy", state='open', assignee='amvdek')

    '''

    _repo_name= "bogumilchilinski/dynpy"

    def __init__(self, token=None, repo_name=None):
        
        from github import Auth

        if token is None:
            pass_code = getpass.getpass('Github token')
            auth = Auth.Token(pass_code)
        else:
            auth = Auth.Token(token)


        g = Github(auth=auth)
        self.g = g

        if repo_name is not None:
            self._repo_name = repo_name
            

    def open(self):
        '''
        This method opens the connection with GitHub API by entering the token.

        Example:

        client = GitHubInterface()

        client.close()

        client.open()

        '''
        
        from github import Auth
        
        if self.g is None:
            pass_code = getpass.getpass('Github token')
            auth = Auth.Token(pass_code)
            g = Github(auth=auth)
            self.g = g
        else:
            pass

    def get_repos_list(self, string=False):
        '''
        This method returns the list of the repositories in form of Repository classes or in string type depending on boolean value of string argument.

        Example:

        client = GitHubInterface()

        client.get_repos_list(string=True)

        '''

        if string == False:
            return list(self.g.get_user().get_repos())
        else:
            repo_list=[]
            for repo in list(self.g.get_user().get_repos()):
                repo_list.append(repo.full_name)
            return repo_list

    def get_repo(self, full_name=None):
        '''
        This method returns the Repository class from GitHub.

        Example:

        client = GitHubInterface()

        client.get_repo(full_name='bogumilchilinski/dynpy')

        '''
        if full_name is None:
            full_name = self._repo_name

        return self.g.get_repo(full_name)

    def get_issues_list(self, repo_name=None, state='all', assignee=None, sort='none', since=None):#, milestone='none',  labels='none', sort='none', direction='none', creator='none', since='none'):

        '''
        This method returns the list of the issues from GitHub Repository in form of Issue classes.

        Example:

        client = GitHubInterface()

        client.get_issues_list(repo_name='bogumilchilinski/dynpy', state='all', assignee='amvdek')
        client.get_issues_list(repo_name='bogumilchilinski/dynpy', state='open', assignee='amvdek')
        client.get_issues_list(repo_name='bogumilchilinski/dynpy', state='closed', assignee='amvdek')

        '''

        if assignee is not None:
            arg_dict =  {'assignee':assignee}
        else:
            arg_dict = {}
            
        if since is not None:
            arg_since =  {'since':since}
        else:
            from datetime import datetime
            since_date = "2024-09-01"
            date_string = since_date + "T00:00:00Z"
            date_object = datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%SZ")
            arg_since = {'since':date_object}

        if repo_name is None:
            repo_name = self._repo_name

        issue_list=[]
        for issue in list(self.g.get_repo(repo_name).get_issues(state=state, sort=sort, **arg_since, **arg_dict)):#, milestone=milestone,  labels=labels, sort=sort, direction=direction, creator=creator)):
            issue_list.append(issue)
        return issue_list


    def issues_list(self, repo_name=None, state='all', assignee=None, sort='none', since=None):
        '''
        This method refers to the method get_issues_list
        
        Example:

        client = GitHubInterface()

        client.issues_list(repo_name='bogumilchilinski/dynpy', state='all', assignee='lsikor')
        
        '''

        if repo_name is None:
            repo_name = self._repo_name

        return self.get_issues_list(repo_name=repo_name, state=state, assignee=assignee, sort=sort, since=since)


    def issues_table(self, repo_name=None, state='all', assignee=None, sort='none', since=None):
        '''
        This method returns the Pandas DataFrame containing Issues.

        Example:

        client = GitHubInterface()

        client.issues_table(repo_name='bogumilchilinski/dynpy', state='open', assignee='lsikor')

        '''

        import pandas as pd
        
        if assignee is not None:
            arg_dict =  {'assignee':assignee}
        else:
            arg_dict = {}
            
        if since is not None:
            arg_since =  {'since':since}
        else:
            from datetime import datetime
            since_date = "2018-09-01"
            date_string = since_date + "T00:00:00Z"
            date_object = datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%SZ")
            arg_since = {'since':date_object}

        if repo_name is None:
            repo_name = self._repo_name

        issue_title_list=[]
        issue_number_list=[]
        issue_assignees_list=[]
        for issue in list(self.g.get_repo(repo_name).get_issues(state=state, sort=sort, **arg_since, **arg_dict)):
            issue_title_list.append(issue.title)
            issue_number_list.append(issue.number)
#             issue_assignees_list.append(issue.assignees)

        data_dict = {'Issue title':issue_title_list, 'Issue number':issue_number_list, }#'Assignees':issue_assignees_list}

        return pd.DataFrame(data_dict)

    def prepare_meeting_issues(self,title=None,last_issue_no=None,guide=None,date=None,time=None,done=False):
        '''
        This method prepare meeting issues by using class MeetingIssueCreator.

        Example:
        
        from dynpy.utilities.creators import GitHubInterface
        title="Issue"
        last_issue_no=999
        client = GitHubInterface(token)
        client.prepare_meeting_issues(title,last_issue_no)

        '''
        if last_issue_no is None:
            last_issue_no = 9999
        return MeetingIssueCreator(title=title,no=last_issue_no+1,guide=guide,date=date,time=time,done=done)
    

    def open_issues(self, state='open', since=None, sort=None):
        '''
        This method returns the list of the Issues.

        Example:

        client = GitHubInterface()

        client.open_issues(state='open', since=None, sort=None)

        '''
        if self.g is not None:
            dp_repo=list(self.g.get_user().get_repos())[1]
            (dp_repo).full_name

            repo = self.g.get_repo(dp_repo.full_name)
            open_issues = repo.get_issues(state=state, since=since, sort=sort)
            for issue in open_issues:
                if 'class that creates a part of a report' in issue.title:
                    print(issue.title.split('`')[1])

            [issue.title.split('`')[1] for issue in open_issues if 'class that creates a part of a report' in issue.title ]

        else:
            return 'Reopen the client'


    def close(self):
        '''
        This method closes the connection with GitHub API.

        Example:

        client = GitHubInterface()

        client.close()

        '''
        if self.g is not None:
            g = self.g
            self.g = g.close()
        else:
            pass


    def str_to_print_issue(self, issue):
        
        title = issue.title
        body = issue.body
        no = issue.number
        asignees = issue.assignees
        id = issue.id
        comms = issue.comments
        labels = issue.labels
        created = issue.created_at
        closed = issue.closed_at

        label_names='/ '

        for label in labels:
            label_names = label_names + label.name + ' / '


        asignees_names='/ '

        for asignee in asignees:
            asignees_names = asignees_names + asignee.login + ' / '

        str_to_print =f'''

        ---------------------\n
        Title: {title} --- number {no} \n
        ---------------------\n
        Assignees: {asignees_names} \n
        ---------------------\n
        Issue created at {created} \n
        ---------------------\n
        Issue closed at {closed}
        ---------------------\n
        Labels: {label_names}
        ---------------------\n
        Description: {body} \n
        ---------------------\n


        '''


        return str_to_print

    def print_issue(self, issue):
        '''
        
        This method refers to the method str_to_print_issue
        
        '''
        return print(self.str_to_print_issue(issue=issue))


    def get_issue(self, repo_name=None, user = None, issue_number = None):
                                        
        #SyntaxError: non-default argument follows default argument, w poprzedniej wersji pojawiał się błąd, tymczasowo dodałem None do pozostałych argumentów. d.stryjewski

        if repo_name is None:
            repo_name = self._repo_name


        since_date = "2018-08-26"
        date_string = since_date + "T00:00:00Z"
        date_object = datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%SZ")

        issues_list = self.get_issues_list(repo_name=repo_name, state='all', assignee=user, sort='none', since=date_object)

        for issue in issues_list:

            if issue.number == issue_number:
                break

        return issue

    def project_summary(self, repo_name=None, issue_number = None):
        '''
        This method returns the list of the related issues with assigness as dataframe.

        Example:

        from dynpy.utilities.creators import GitHubInterface
        client = GitHubInterface()
        client.project_summary(issue_number = 954)

        '''
        
        import pandas as pd
        
        if repo_name is None:
            repo_name = self._repo_name
            
        if issue_number is None:
            issue_number = 954

        issues_list = self.get_issues_list(repo_name=repo_name, state='all', assignee="*", sort='none')
        issue_title_list=[]
        issue_number_list=[]
        issue_assignees_list=[]
        
        for issue in issues_list:

            if issue.number == issue_number:
                issue_title_list.append(issue.title)
                issue_number_list.append(issue.number)
                asignees = issue.assignees
                asignees_names=''
                for asignee in asignees:
                    asignees_names = asignees_names + asignee.login + ' / '
                issue_assignees_list.append(asignees_names)
            elif "issue #" + str(issue_number) + " related" in issue.title:
                issue_title_list.append(issue.title)
                issue_number_list.append(issue.number)
                asignees = issue.assignees
                asignees_names=''
                for asignee in asignees:
                    asignees_names = asignees_names + asignee.login + ' / '
                issue_assignees_list.append(asignees_names)
            
        data_dict = {'Issue number':issue_number_list, 'Issue title':issue_title_list, 'Assignees':issue_assignees_list}

        return pd.DataFrame(data_dict)

class ClassLister:
    
    '''
    This is temporary solution, working on better one.
    
    Example of call code:
    
    ###############################
    import dynpy
    from dynpy.utilities.report import ReportText

    dir_str = 'dynpy.utilities.components.guides'

    ClassLister(dir_str).printer()
    ###############################
    '''
    
    import dynpy
    from datetime import date, time

    
    def __init__(self, directory):
        self.directory = directory
        
        import_str = f'from {directory} import *'
        exec(import_str) #exec i eval to dwie funkcje pozwalające na wykonanie komendy podanej jako string
    
    def submodgetter(self): 
        '''
        metoda zbiera liste sbmodułów z wskazanej scieżki
        
        '''
        import os.path, pkgutil
        
        pkgpath_str = f"os.path.dirname({self.directory}.__file__)"
        
        pkgpath = eval(pkgpath_str)
        list_of_sub_modules = [name for _, name, _ in pkgutil.iter_modules([pkgpath])]

        return list_of_sub_modules
    
    def classlistgetter(self, submod):
        
        '''
        metoda zbiera liste klass w podanym submodule
        
        init: submod
        '''
        
        import inspect

        import_str = f"from {self.directory}.{submod} import *"
        command_str = f"inspect.getmembers({self.directory}.{submod}, predicate=inspect.isclass)" #ta komenda zbiera liste wszystkich klas istniejących w danym submodule, niestety sa tu też klasy nas nie interesujące np. importy klas z numpy czy pandasa etc, dlatego dalej ta lista jest filtrowana
        
        try:
            import_eval = exec(import_str)
            members = eval(command_str)
        except:
                pass
            
        #members = eval(command_str)

        tmp_list = []

        for i in range(len(members)):
            if "__class__" in members[i][0]: #ten warunek doda klase do listy w przypadku gdy submoduł ma ta samą nazwe co klasa sama w sobie 
                tmp_list.append(submod)
            elif f'{self.directory}.{submod}' in str(members[i][1]):
                tmp_list.append(members[i][0])

        return tmp_list
    
    def printer(self):
        lst = []
        lst = self.submodgetter()
        
        cnt = 0
        
        for submod in lst:    
            class_lst = self.classlistgetter(submod)
            size = len(class_lst)
            
            if size != 0:
                display(ReportText(f'# {submod}'))

            for cl in class_lst:
                    display(ReportText(f'* {cl}'))
                    cnt += 1
        display(ReportText(f'Total class count is:{cnt}'))
    
    
    def initupdater(self):
        intro_str = f'''"""
This module provides the examples of mechanical models being practically used in calculations

Last update was done on {date.today()}

"""

import importlib

'''

        from datetime import date, time
        
        f = open('./output/test.py', 'w')
        f.write(intro_str)
        
        lst = []
        lst = self.submodgetter()
        for submod in lst:    
            class_lst = self.classlistgetter(submod)
            f.write(f'from .{submod} import ')

            size = len(class_lst)
            i = 1

            for cl in class_lst:

                if i < size:
                    f.write(f'{cl}, ')
                else:
                    f.write(f'{cl}')

                i += 1 

            f.write('\n\n')

        f.close()
        
    def get_init_file_content(self):
        from datetime import date, time
        from dynpy.utilities.report import ObjectCode
        
        output_str = f'#Last update was done on {date.today()} \n\n'
        
        lst = []
        lst = self.submodgetter()
        for submod in lst:    
            class_lst = self.classlistgetter(submod)
            output_str = output_str + f'from .{submod} import '

            size = len(class_lst)
            i = 1

            for cl in class_lst:

                if i < size:
                    output_str = output_str + f'{cl}, '
                else:
                    output_str = output_str + f'{cl}'

                i += 1 

            output_str = output_str + '\n\n'
        return output_str



def list_of_guides_prime():
    md_str='Lista poradników: \n\n'
    for name, obj in inspect.getmembers(document):
        if inspect.isclass(obj) and 'Guide' in name:
                if obj is not Guide:
                    md_str=md_str+f'\t\t - {name} \n\n'
    return md_str



class ModuleStructure:
    
    '''
Class was created to support development work on dynsys.

Example usage:
===========================================================================================
import dynpy
from dynpy.utilities.report import ReportText

example = ModuleStructure(dynpy.utilities.components)
example.get_classes('Code') #Creates a list of components. Optional keyword used for filtering
example.printer() #Prints list in user friendly format
example.tree() #Prints created list in tree format
===========================================================================================
    '''
    
    def __init__(self, directory, depth = False):
        
        import dynpy
        from datetime import date, time
        import inspect, os, types
        from dynpy.utilities.report import ReportText
        
        self.full_list = []
        self.tree_str = []
        self.depth = depth

        if isinstance(directory, types.ModuleType):
            s = str(directory)
            self.directory = s[s.find("module '")+len("module '"):s.rfind("' from ")]
        elif isinstance(directory, str):
            self.directory = directory
        
        import_str = f'from {self.directory} import *'
        exec(import_str)

    def submodgetter(self):
        '''
Method collects a list of submodules in the given module
        
init: None
        '''
        import inspect
        import dynpy
        import dynpy.models
        
        tmp_list = eval(f"inspect.getmembers({self.directory}, predicate=inspect.ismodule)")
        
        self.mod_list = []
        
        for i in range(len(tmp_list)):
            s = str(tmp_list[i][1])

            tmp = s[s.find("module '")+len("module '"):s.rfind("' from ")]
            
            if f"{self.directory}" in tmp: 
                self.mod_list.append(f'{self.directory}')
        tmp_list.clear()
        
        if self.depth == 1:
            self.mod_list = sorted(self.mod_list)
            return self.mod_list
        
        for element in self.mod_list:
            exec(f'from {element} import *')
            tmp_list = eval(f"inspect.getmembers({element}, predicate=inspect.ismodule)")
            if tmp_list is not None:
                for i in range(len(tmp_list)):
                    s = str(tmp_list[i][1])
                    tmp = s[s.find("module '")+len("module '"):s.rfind("' from ")]
                    if (f"{self.directory}" in tmp) and (tmp not in self.mod_list):
                        self.mod_list.append(f'{tmp}')
        
        self.mod_list = sorted(self.mod_list)
        
        return self.mod_list

    def _classlistgetter(self, submod, keyword):
        '''
Method collects a list of classes in the given submodule
keyword - optional argument, keyword by which the classes are searched for
        
init: submod,keyword
        '''
        
        import inspect

        import_str = f"from {submod} import *"
        command_str = f"inspect.getmembers({submod}, predicate=inspect.isclass)" 
        
        import_eval = exec(import_str)
        members = eval(command_str)

        tmp_list = []

        for i in range(len(members)):
            if "__class__" in members[i][0]: 
                if (keyword != None) and (keyword in members[i][0]):
                    tmp_list.append((f'{self.directory}.submod', submod))
                elif keyword == None:
                    tmp_list.append((f'{self.directory}.submod', submod))
            if f'{submod}' in str(members[i][1]):
                if (keyword != None) and (keyword in members[i][0]):
                    s = str(members[i][1])
                    tmp = s[s.find("<class '")+len("<class '"):s.rfind(f".{members[i][0]}'>")]
                    tmp_list.append((tmp,members[i][0]))
                elif keyword == None:
                    s = str(members[i][1])
                    tmp = s[s.find("<class '")+len("<class '"):s.rfind(f".{members[i][0]}'>")]
                    tmp_list.append((tmp,members[i][0]))
                    
        return tmp_list
    
    def get_classes(self, keyword = None):
        
        '''
Method collects a list of classes and parent submodule directory in the given module
keyword - optional argument, keyword by which the classes are searched for
        
init: keyword
        '''
        
        self.submodgetter()
        
        self.full_list = []
        
        cnt = 0
        
        if not self.mod_list:
            class_lst = self._classlistgetter(self.directory, keyword)
            
            if len(class_lst) != 0:
                for cl in class_lst:
                        self.full_list.append(cl)
        
        for submod in self.mod_list:    
            class_lst = self._classlistgetter(submod, keyword)
            size = len(class_lst)
            
            if size != 0:
                for cl in class_lst:
                        self.full_list.append(cl)
                        
                        cnt += 1
        self.full_list = list(set(self.full_list))
        
        return self.full_list
    
    def printer(self, keyword = None):
        
        '''
Method prints a list of classes and parent submodule directory in the given module
keyword - optional argument, keyword by which the classes are searched for
        
init: keyword
        '''
        
        if not self.full_list:
            self.get_classes(keyword)
        
        i = 0
        a = 0
        
        submod_prev = None
        class_name = self.full_list[0][1]
        
        for element in self.full_list:
            if submod_prev == self.full_list[i][0]:
                display(ReportText(f'* {self.full_list[i][1]}'))
                submod_prev = self.full_list[i][0]
            else:
                display(ReportText(f'### {self.full_list[i][0]}'))
                display(ReportText(f'* {self.full_list[i][1]}'))
                
                submod_prev = self.full_list[i][0]

            i = i+1 
        
        display(ReportText(f'Total class count is: {i}'))
        
    def get_module_tree(self, keyword = None):
        
        '''
Method prints a tree of classes
keyword - optional argument, keyword by which the classes are searched for
        
init: keyword
        '''
        
        if not self.full_list:
            self.get_classes(keyword)
        
        self.tree_str = TreeNode.build_tree(self.full_list)
        
        return self.tree_str
    
    def as_reporttext(self, keyword = None):
        '''
Method format tree for ReportText and Markdown 
keyword - optional argument, keyword by which the classes are searched for        
        '''
        from dynpy.utilities.report import ReportText
        
        if not self.full_list:
            self.get_classes(keyword)
        
        return ReportText(TreeNode.as_reporttext(self.full_list).replace('\n', "<br />"))
    
    def as_objectcode(self, keyword = None):
        '''
Method format tree for ObjectCode
keyword - optional argument, keyword by which the classes are searched for        
        '''
        from dynpy.utilities.report import ObjectCode
        
        if not self.full_list:
            self.get_classes(keyword)
            
        if not self.tree_str:
            self.tree_str = TreeNode.build_tree(self.full_list)
        
        return ObjectCode(self.tree_str)
    
    def __repr__(self):
        return self.get_module_tree()
    
    def _get_init(self):
        from datetime import date, time
        from dynpy.utilities.report import ObjectCode

        output_str = f'''#Last update was done on {date.today()}  '''
        
        self.get_classes()
        self.full_list = list(set(self.full_list))        
        self.full_list = sorted(self.full_list)
        
        if len(self.full_list) == 0:
            try:
                output_str = ClassLister(self.directory).get_init_file_content()
            except:
                pass

        prev = []
        
        for element in self.full_list:
            if element[0] != prev:
                
                prev = element[0]
                output_str = output_str[:-2]
                output_str = output_str + '\n\n'
                
                s = prev
                s = s[s.find(self.directory)+len(self.directory):s.rfind("")]
                if "'" in s:
                    s = s[:-1]
                output_str = output_str + f'from {s} import '

                output_str = output_str + f'{element[1]}, '
            else:
                output_str = output_str + f'{element[1]}, '

        if output_str[-2] == ',':
            output_str = output_str[:-2]
            
        return output_str
    
    @staticmethod
    def get_init_file_content(dir_list = None):
        
        from dynpy.utilities.report import ReportText, ObjectCode
        
        if dir_list == None:
            dir_list = [
    'dynpy',
    'dynpy.models',
    'dynpy.models.control',
    'dynpy.models.electric',
    'dynpy.models.mechanics',
    'dynpy.models.odes',
    'dynpy.solvers',
    'dynpy.utilities',
    'dynpy.utilities.components',
    'dynpy.utilities.components.guides',
    'dynpy.utilities.components.guides.development',
    'dynpy.utilities.components.guides.github',
    'dynpy.utilities.components.guides.pandas',
    'dynpy.utilities.components.guides.reporting',
    'dynpy.utilities.components.guides.systems',
    'dynpy.utilities.components.mech',
    'dynpy.utilities.components.ode',
    'dynpy.utilities.components.miscellaneous',
    'dynpy.utilities.documents',
                        ]
        for element in dir_list:
            display(ReportText(f'This is __init__.py content for {element} module'))
            display(ObjectCode(__class__(element)._get_init()))
    
    @staticmethod
    def get_import(keyword , as_type = 'ObjectCode'):
        from dynpy.utilities.report import ObjectCode
        
        tmp = __class__('dynpy').get_classes(keyword)
        lst = []

        if len(tmp) == 0:
            return "Couldn't get import"
        else:
            for element in tmp:
                lst.append(f'from {element[0]} import {element[1]}')
            
            if as_type == 'ObjectCode':
                lst = '\n'.join(lst)
                return ObjectCode(lst)
            elif as_type == str:
                return lst

class TreeNode:
    '''
Class created to present table of directories in tree format known from commandline tool in Unix systems.

To call tree use bellow call code:

    TreeNode.build_tree(full_list, separator, rootnode)

    Where:
        full_list - list, where each node is separated by some symbol, for example: dynpy.utilities.components
        separator (optional, by defualt it is '.') - symbol used to separate each node '.' in above example 
        rootnode (optional, by defualt it is 'dynpy'- node that starts creating tree, for example 'dynpy'
    '''
    def __init__(self, name, parent = None):
        self.parent = parent
        self.name = name
        self.children = []
        self.str_out = []

    def add_child(self, node):
        self.children.append(node)
        return node

    def print(self, is_root = True, syntax = True):
        '''
Method is used to create tree string.

init: 
syntax - is used to define tab sign, Object code accept normal spaces, Markdown and ReportText require &nbsp; instead of tab
        '''
        if syntax is True:
            pre_0 = "    "
            pre_1 = "│   "
            pre_2 = "├── "
            pre_3 = "└── "
        elif syntax is False:
            pre_0 = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
            pre_1 = "│&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
            pre_2 = "├──&nbsp;"
            pre_3 = "└──&nbsp;"


        tree = self
        prefix = pre_2 if tree.parent and id(tree) != id(tree.parent.children[-1]) else pre_3

        while tree.parent and tree.parent.parent:
            if tree.parent.parent and id(tree.parent) != id(tree.parent.parent.children[-1]):
                prefix = pre_1 + prefix
            else:
                prefix = pre_0 + prefix

            tree = tree.parent

        if is_root:
            self.str_out.append(self.name)
        else:
            self.str_out.append(prefix + self.name)

        for child in self.children:
            self.str_out.append(child.print(is_root = False, syntax = syntax))
            
        return self.str_out

    @staticmethod
    def find_and_insert(parent, edges):
        if not edges:
            return

        match = [tree for tree in parent.children if tree.name == edges[0]]

        tree = match[0] if match else parent.add_child(TreeNode(edges[0], parent))

        __class__.find_and_insert(tree, edges[1:])
        
    @staticmethod
    def unpack(lst_to_unpack):
        '''
This method is a helper to unpack nested list returned by print method.
Method will return string ready to use by ObjectCode.

init: list returned from print() method
        '''
        
        unpacked_lst = []
        for element in lst_to_unpack:
            if isinstance(element, list):
                unpacked_lst.extend(__class__.unpack(element))
            else:
                unpacked_lst.append(element)
                unpacked_lst.append('\n')
            lst = ''.join(unpacked_lst)
        return lst
    
    @classmethod
    def build_tree(cls, full_list, separator = '.', rootnode = 'dynpy'):
        
        '''
Method created to build full tree from declared list

init: full_list, separator, rootnode

        '''
        
        lst_comp = []
        i = 0

        while i != len(full_list):
            t_str = f'{full_list[i][0]}.{full_list[i][1]}'
            lst_comp.append(t_str)
            i=i+1
            
        root = __class__(rootnode)

        for element in lst_comp:
            root.find_and_insert(root, element.split(separator)[1:])
        tree_str = root.print()
        
        cls.tree_comp = __class__.unpack(tree_str)
        
        return cls.tree_comp
    
    
    @classmethod
    def as_reporttext(cls, full_list, separator = '.', rootnode = 'dynpy'):
        '''
Staticmethod return tree string in ReportText format

Init: tree_comp - string returned by build_tree method
        '''
        lst_comp = []
        i = 0

        while i != len(full_list):
            t_str = f'{full_list[i][0]}.{full_list[i][1]}'
            lst_comp.append(t_str)
            i=i+1
            
        root = __class__(rootnode)

        for element in lst_comp:
            root.find_and_insert(root, element.split(separator)[1:])
        tree_str = root.print(is_root = True, syntax = False)
        
        comp = __class__.unpack(tree_str)
        
        return comp
        
        
def list_of_guides():

    #dir_str = 'dynpy.utilities.documents'

    #return [guide_cls for guide_cls in ModuleStructure(dir_str).get_classes() if 'guides' in guide_cls[0]]
    
    return ModuleStructure(dynpy.utilities.documents.guides).get_classes()

def list_of_components():

    dir_str = 'dynpy.utilities.components'
    
    return ModuleStructure(dir_str).get_classes()

def list_of_mechanical_systems():

    dir_str = 'dynpy.models.mechanics'

    return ModuleStructure(dir_str).get_classes()

def list_of_dynamic_systems():
        
    dyn_lst = []
         
    dyn_lst = ModuleStructure('dynpy.models.mechanics').get_classes() + ModuleStructure('dynpy.models.electric').get_classes()   
 
    return dyn_lst


advanced_modeling_schedule_code_str = """Następujące zadania mają na celu przygotowanie projektu wybranego przez studentów układu dynamicznego. Konieczne jest zaplanowanie wykonania kamieni milowych w terminie do {date} i w czasie {time}, celem utrzymania ciągłości realizacji projektu:

- [{tic}] #{issue_no+1}

- [{tic}] #{issue_no+2}

- [{tic}] #{issue_no+3}

- [{tic}] #{issue_no+4}

- [{tic}] #{issue_no+5}

- [{tic}] #{issue_no+6}

- [{tic}] #{issue_no+7}

"""
 

advanced_modeling_intro_code_str = """Celem jest przygotowanie toku postępowania, dzięki któremu możliwe będzie sprawne przygotowanie projektu zaawansowanego modelu dynamicznego:

 FAZA I - WSTĘPNA - 1 tydzień (ZESPÓŁ/ 1 osoba)

- [{tic}] Określenie wymagań oraz oczekiwań do projektu, w jakiej dziedzinie chcemy wykonać pracę:
                mechaniczna np. przekładnie, silniki, 
                elektryczna np. akumulatory, filtry, uklady sterowania
                budowlana np. dynamiczne eliminatory drgań, żurawie 
                dynamiczne np. zawieszenia, 
                kinematyczne np. ruch "pojazdu" po trasie wraz z jego charakteryzacją

- [{tic}] Wybór rzeczywistego obiektu, mechanizmu lub zjawiska, na bazie którego wykonany zostanie model.

- [{tic}] Prezentacja przykładowego rzeczywistego modelu doświadczalnego, który nawiązuje do modelu symulacyjnego

- [{tic}] Wybór metodyki stworzenia modelu dynamicznego,

- [{tic}] Określenie założeń do modelu, stopnia zaawansowania oraz konkretnych wyników jakie chcemy uzyskać

"""

advanced_modeling_basics_code_str = """Celem jest przygotowanie toku postępowania, dzięki któremu możliwe będzie sprawne przygotowanie projektu zaawansowanego modelu dynamicznego:

 FAZA II - PRZYGOTOWANIA PRACY - 2 tygodnie (1 osoba)

- [{tic}] Przegląd aktualnego stanu wiedzy oraz zawartości środowiska DynPy, celem rozpoznania możliwości,

- [{tic}] Przygotowanie dokumentu, w którym zaprezentowane będą podstawowe klasy i metody na bazie których możliwe będzie wykonanie projektu oraz raportu,

- [{tic}] Wyszukanie grafiki obiektu, mechanizmu lub zjawisa na podstawie którego należy wykonać rysunek schematyczny (np. w SolidWorks/Autocad/TiKz) 

- [{tic}] Przeprowadzenie całkowitej charakterystyki modelu (LSS, współrzędne, wiezy,  energie, rownania ruchu)

"""

advanced_modelling_modelling_code_str = """Celem jest przygotowanie toku postępowania, dzięki któremu możliwe będzie sprawne przygotowanie projektu zaawansowanego modelu dynamicznego:

 FAZA III - WYKONANIA MODELU - 3 tygodnie (1 osoba)

- [{tic}] Wybór modelu z biblioteki DynPy, który przypomina zdefiniowany model w poprzedniej fazie,

- [{tic}] Przygotowanie, bazując na modelu,  wszystkich potrzebnych do analizy równań, 

- [{tic}] Dobranie odpowiednich danych i parametrów do wybranego modelu symulacyjnego,

- [{tic}] Przeprowadzenie weryfikacji poprawności zaprogramowanych funckji,

"""

advanced_modeling_simulation_code_str = """Celem jest przygotowanie toku postępowania, dzięki któremu możliwe będzie sprawne przygotowanie projektu zaawansowanego modelu dynamicznego:

 FAZA IV - WYKONANIA SYMULACJI - 2 tygodnie (1 osoba) 

- [{tic}] Przygotowanie odpowiednich funckji do wykonania symulacji analitycznych i numerycznych,

- [{tic}] Wykonanie symulacji z wykorzystaniem analitycznych rozwiązań równanań ruchu,

- [{tic}] Przeprowadzenie symulacji z wykorzystaniem modelu znumeryzowanego,

- [{tic}] Przeprowadzenie analizy poprawności dobranych danych oraz wykonanie ewentualnej modyfikacji,

- [{tic}] Stworzenie matrycy porównującej wyniki obu symulacji,

- [{tic}] (+) Wykonanie obliczeń wielkości innych niż "x,v,a",

"""

advanced_modeling_analysis_code_str = """Celem jest przygotowanie toku postępowania, dzięki któremu możliwe będzie sprawne przygotowanie projektu zaawansowanego modelu dynamicznego:

 FAZA V - ANALIZY WYNIKÓW - 1 tydzień (1 osoba)

- [{tic}] Zaprezentowanie wyników w formie zbioru tabel

- [{tic}] Prezentacja wyników w formie wykresów wraz

- [{tic}] Wykonanie analizy wyników i zdefiniowanie zauważonych zjawisk

"""

advanced_modeling_report_code_str = """Celem jest przygotowanie toku postępowania, dzięki któremu możliwe będzie sprawne przygotowanie projektu zaawansowanego modelu dynamicznego:

 FAZA VI - WYKONANIA RAPORTU KOŃCOWEGO - 2 tygodnie (1 osoba)

- [{tic}] Stworzenie krótkiego opisu aktualnego stanu wiedzy wraz z opisem idei modelowania (FAZA II),

- [{tic}] Zaprezentowanie wybranego modelu rzeczywistego oraz jego interpretacji fizycznej (FAZA I),

- [{tic}] Prezentacja wykorzystanych równań charakteryzujących obiekt (FAZA III),

- [{tic}] Przygotowanie opisu stosowanego programu (FAZA III),

- [{tic}] Wykonanie tabel i wykresów oraz ich opis (FAZA IV, V)

- [{tic}] Sformułowanie wniosków (FAZA VI)

"""

advanced_modeling_schedule_code_str_en = """
The aim of the issue is to manage the process of project implementation.
The following activities are to do:

- [{tic}] assignment of tasks;

- [{tic}] presentation of requirements that need to be fulfilled (code, issues, other things);

- [{tic}] ???

- [{tic}] presentation of performed activities;

The tasks to complete are as follows:

- [{tic}] #{issue_no+1}

- [{tic}] #{issue_no+2}

- [{tic}] #{issue_no+3}

- [{tic}] #{issue_no+4}

- [{tic}] #{issue_no+5}

- [{tic}] #{issue_no+6}

- [{tic}] #{issue_no+7}

"""


advanced_modeling_intro_code_str_en = """The goal is preparation of work environment and initial content (draft) version of the report:

 PHASE I - PRELIMINARY - 1 week (TEAM/ 1 person)

[{tic}] Preparation of folder structure, necessary files and preleminary content report,

[{tic}] Definition of the requirements and expectations for the project, determination of the field in which the work will be conducted: mechanical, electrical, structural, dynamic, kinematic along a route, along with its characterization,

[{tic}] Selection of a real object, mechanism, or phenomenon on which the model will be based,

[{tic}] Presentation of an example of a real experimental model that relates to the simulation model,

[{tic}] Definition of the assumptions for the model, the level of advancement, and the specific results we aim to achieve.

"""

advanced_modeling_basics_code_str_en = """The goal is research on current state of art about the topic and methodology needed for project:

 PHASE II - PROJECT PREPARATION - 2 weeks (1 person)

[{tic}] Revision of the current state of knowledge,

[{tic}] Revision of the DynPy environment to assess its capabilities,

[{tic}] Selection of a methodology for creating the dynamic model,

[{tic}] Preparation of a document presenting the fundamental classes and methods that will form the basis of the project and report,

[{tic}] Searching for graphics of the object, mechanism, or phenomenon on which the schematic drawing will be based (e.g., in SolidWorks/AutoCAD/TiKz),

[{tic}] Conduction a complete characterization of the model (LSS, coordinates, constraints, energies, equations of motion).

"""

advanced_modelling_modelling_code_str_en = """The goal is creating the mathematical model of investigated object:

 PHASE III - MODEL DEVELOPMENT - 3 weeks (1 person)

[{tic}] Selection of a model from the DynPy library that resembles the model defined in the previous phase,

[{tic}] Praparation, based on the model, of all the necessary equations for the analysis,

[{tic}] Selection of appropriate data and parameters for the chosen simulation model,

[{tic}] Conduction verification process of the programmed functions to ensure correctness.

"""

advanced_modeling_simulation_code_str_en = """The goal is using the model to perform all necessary simulations and got valuable data:

 PHASE IV - SIMULATION EXECUTION - 2 weeks (1 person)

[{tic}] Preparation of appropriate functions for conducting analytical and numerical simulations,

[{tic}] Execution of simulations using analytical solutions of the equations of motion,

[{tic}] Conduction of simulations using the numerically discretized model,

[{tic}] Analysing accuracy of the selected data and making any necessary adjustments,

[{tic}] Creation of a matrix comparing the results of both simulations,

[{tic}] (+) Performation of calculations for quantities other than "x, v, a".

"""

advanced_modeling_analysis_code_str_en = """The goal is validation and proper visualization of all acquired data:

 PHASE V - RESULTS ANALYSIS - 1 week (1 person)

[{tic}] Presentation of the results in the form of a set of tables,

[{tic}] Displaying of the results in the form of graphs,

[{tic}] Performing of an analysis of the results and identification of the observed phenomena.

"""

advanced_modeling_report_code_str_en = """The goal is revision of all steps of the research and creation of the final report.

 PHASE VI - FINAL REPORT PREPARATION - 2 weeks (1 person)

[{tic}] Creation of a brief summary of the current state of knowledge along with a description of the modeling concept (PHASE II),

[{tic}] Presentation of the selected real model and its physical interpretation (PHASE I),

[{tic}] Presentation  of the equations used to characterize the object (PHASE III),

[{tic}] Preparation a description of the applied software (PHASE III),

[{tic}] Preparation of tables and graphs along with their descriptions (PHASE IV, V),

[{tic}] Formulation of conclusions (PHASE VI).

"""

class ResearchProjectIssueCreator:
    
    _title = 'name of the research project'
    _issue_no = 567
    _guide = ODESystemOverviewReport #### ResearchProjectGuidelines
    _time = '14:30' ## 
    _date = '2024.10.10' ##
    _lang = 'en'
    _project_type = 'research'

    def __init__(self,title=None,no=None,guide=None,date=None,time=None,done=False,lang=None,*args,**kwargs):
        if title is not None: self._title = title
        if no is not None: self._issue_no = no
        if guide is not None: self._guide = guide
        if date is not None: self._date= date
        if time is not None: self._time = time
        if lang is not None: self._lang = lang
        self._done = done
        
    
    def _get_elems_dict(self):

        if self._done: tic = 'x'
        else: tic = ' '

        guide_class = self._guide

        elems_dict = {
                'title':self._title,
                'issue_no':self._issue_no,
                'issue_no+1':self._issue_no+1,
                'issue_no+2':self._issue_no+2,
                'issue_no+3':self._issue_no+3, #### 7 ISSUE
                'issue_no+4':self._issue_no+4,
                'issue_no+5':self._issue_no+5,
                'issue_no+6':self._issue_no+6,
                'issue_no+7':self._issue_no+7,
                'guide_class_module':guide_class.__module__,
                'guide_class_name':guide_class.__name__,
                'date':self._date,
                'time':self._time,
                'tic':tic,
                'type':self._project_type,
                }
        
        return elems_dict
    
    def get_issues_titles(self):
        
        
        # Issue no. #{issue_no}
        ## Preparation of meeting and guide for {title}

        issue_no = self._issue_no
        title = self._title
        lang = self._lang
        project_type = self._project_type
        
        
        if lang == 'en':


            titles_dict =   {
                        'schedule': f'{project_type.capitalize()} project on {title}', ### 
                        'intro': f'Preparation of the preleminary content report for {project_type} on {title} (issue #{issue_no} related)', ### 
                        'basics': f'Investigation of state of the art and research methodology for {title} (issue #{issue_no} related)', ### 
                        'modelling': f'Modelling of the investigated object for the {title} (issue #{issue_no} related)', ### 
                        'simulation': f'Performing simulations of investigated case in the {project_type} {title} (issue #{issue_no} related)', ### 
                        'analysis': f'Analysis and validation of obtained results in the {project_type} on {title} (issue #{issue_no} related)', ### 
                        'report': f'Ending revision of a report covering all phases of {project_type} on {title} (issue #{issue_no} related)', ###
                        }
            
        else:
            titles_dict =   {
                        'schedule':f'Prezentacja harmonogramu prac na zajęciach z przedmiotu {title}', ### 
                        'intro':f'Wprowadzenie do środowiska DynPy do wykonania prac z przedmiotu {title} (powiązane z issue #{issue_no})', ### 
                        'basics':f'Przygotowanie podstawowych informacji o modelu oraz zaproponowanie koncepcji stanowiska badawczego z przedmiotu {title} (powiązane z issue #{issue_no})', ###
                        'modelling':f'Wykonanie modelu symulacyjnego z wybranego zagadnienia z przedmiotu {title} (powiązane z issue #{issue_no})', ###
                        'simulation':f'Przeporwadzenie symulacji rozważanego modelu z {project_type} o {title} (powiązane z issue #{issue_no})', ###
                        'analysis':f'Analiza i zatwierdzenie otrzymanych wyników z {project_type} o {title} (powiązane z issue #{issue_no})', ###
                        'report':f'Końcowa rewizja raportu z wszystkich faz {project_type} o {title} (powiązane z issue #{issue_no})', ###
                        }

        return titles_dict
        
    def get_issue_str(self):
        elems_dict = self._get_elems_dict()
        issue_no = self._issue_no
        
        titles = self.get_issues_titles()
        
        schedule_issue_dict = self.get_schedule_issue_dict()
        intro_issue_dict = self.get_intro_issue_dict()
        basics_issue_dict = self.get_basics_issue_dict()
        modelling_issue_dict = self.get_modelling_issue_dict()
        simulation_issue_dict = self.get_simulation_issue_dict()
        analysis_issue_dict = self.get_analysis_issue_dict()
        report_issue_dict = self.get_report_issue_dict()
        
        
        issue_code_str = (f'# Issue no. #{issue_no} \n '+
                        '## '+schedule_issue_dict['title'] + '\n ' +
                        schedule_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+1} \n '+
                        '## '+ intro_issue_dict['title'] + '\n ' +
                        intro_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+2} \n '+
                        '## '+basics_issue_dict['title'] + '\n ' +
                        basics_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+3} \n '+
                        '## '+modelling_issue_dict['title'] + '\n ' +
                        modelling_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+4} \n '+
                        '## '+simulation_issue_dict['title'] + '\n ' +
                        simulation_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+5} \n '+
                        '## '+analysis_issue_dict['title'] + '\n ' +
                        analysis_issue_dict['body'] + '\n ' +
                        f'# Issue no. #{issue_no+6} \n '+
                        '## '+report_issue_dict['title'] + '\n ' +
                        report_issue_dict['body'])
        
        return issue_code_str


    def get_schedule_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang

        if lang == 'en':
            return {'title':titles['schedule'],'body':advanced_modeling_schedule_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['schedule'],'body':advanced_modeling_schedule_code_str.format(**elems_dict)}

    def get_intro_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang

        if lang == 'en':
            return {'title':titles['intro'],'body':advanced_modeling_intro_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['intro'],'body':advanced_modeling_intro_code_str.format(**elems_dict)}

    def get_basics_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang

        if lang == 'en':
            return {'title':titles['basics'],'body':advanced_modeling_basics_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['basics'],'body':advanced_modeling_basics_code_str.format(**elems_dict)}
    
    def get_modelling_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang

        if lang == 'en':
            return {'title':titles['modelling'],'body':advanced_modelling_modelling_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['modelling'],'body':advanced_modelling_modelling_code_str.format(**elems_dict)}
    
    def get_simulation_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang
        
        if lang == 'en':
            return {'title':titles['simulation'],'body':advanced_modeling_simulation_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['simulation'],'body':advanced_modeling_simulation_code_str.format(**elems_dict)}
    
    def get_analysis_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang
        
        if lang == 'en':
            return {'title':titles['analysis'],'body':advanced_modeling_analysis_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['analysis'],'body':advanced_modeling_analysis_code_str.format(**elems_dict)}
    
    def get_report_issue_dict(self):
        elems_dict = self._get_elems_dict()
        titles = self.get_issues_titles()
        lang = self._lang

        if lang == 'en':
            return {'title':titles['report'],'body':advanced_modeling_report_code_str_en.format(**elems_dict)}
        else:
            return {'title':titles['report'],'body':advanced_modeling_report_code_str.format(**elems_dict)}
    
    def _repr_markdown_(self):
        return self.get_issue_str()

    def publish_on_github(self,repository):
        
        guide_label=repository.get_label('guidance')
        
        issue_schedule = repository.create_issue(**self.get_schedule_issue_dict(),labels=[guide_label]#,assignees=wykon
                                )
        
        issue_intro = repository.create_issue(**self.get_intro_issue_dict(),labels=[guide_label]#,assignees=wykon
                                )
        
        issue_basics = repository.create_issue(**self.get_basics_issue_dict(),labels=[guide_label]#,assignees=wykon
                                )
        
        issue_modelling = repository.create_issue(**self.get_modelling_issue_dict(),labels=[guide_label],assignee='bogumilchilinski')
        
        issue_simulation = repository.create_issue(**self.get_simulation_issue_dict(),labels=[guide_label]#,assignees=wykon
                                )
        
        issue_analysis = repository.create_issue(**self.get_analysis_issue_dict(),labels=[guide_label]#,assignees=wykon
                                )
        
        issue_report = repository.create_issue(**self.get_report_issue_dict(),labels=[guide_label]#,assignees=wykon
                                )
        
        return issue_schedule, issue_intro, issue_basics, issue_modelling, issue_simulation, issue_analysis, issue_report
    
class PDF_link:
    '''
Class create hyperlink to PDF file, that will open document in new browser tab.

====================================================================================================
Arguments:
    
    document - instance of class from dynpy.utilities.templates.document like Guide, WUTthesis etc.
====================================================================================================
Use example:

    from dynpy.utilities.templates.document import WutThesis, Guide

    doc = WutThesis('./output/file')
    doc.generate_pdf(clean_tex=True)

    PDF_link(doc)
====================================================================================================
    '''
    def __init__(self, document):
        import IPython
        from IPython.display import display_pdf
        path = f'{document.default_filepath}'
        with open((document.default_filepath + '.pdf'), "rb") as f:
            display_pdf(f.read(),raw=True)

        f.close()


from sympy import *
from dynpy.solvers.linear import ODESystem
from dynpy.solvers.perturbational import MultiTimeScaleSolution
# from dynpy.models.mechanics.trolleys import MDoFTMD1

class LinearSecondOrderMSM(ODESystem):
    
    @classmethod
    def from_reference_data(cls):
        F=Symbol('F')
        c=Symbol('c',positive=True)
        k=Symbol('k',positive=True)
        eps=Symbol('varepsilon')
        t=Symbol('t')
        z=Function('z')(t)
        a0=Symbol('a_0',positive=True)
        a1=Symbol('a_1',positive=True)
        omg=Symbol('omega',positive=True)
        m=Symbol('m',positive=True)
        eom=-F+c*z.diff(t)+k*(eps*(a0*sin(omg*t)+a1*sin(2*omg*t)+1)*z+m*z.diff(t,2))


        odes = cls(eom,dvars=z,ivar=t, order=2)._as_msm()
        
        return odes
    
class MDoFTMDMSM(MultiTimeScaleSolution):
    
    @classmethod
    def from_reference_data(cls):

        t=Symbol('t')
        xb=Function("x_b")(t)
        xe=Function("x_e")(t)
        eps= Symbol('varepsilon',positive=True)
        delta =  Symbol('delta',positive= True)
        delta1 =  Symbol('delta_1',positive= True)
        delta2 =  Symbol('delta_2',positive= True)
        omg1 =  Symbol('omega_1',positive= True)
        omg2 =  Symbol('omega_2',positive= True)

        omg =  Symbol('Omega',positive= True)

        K_mat = Matrix([[omg**2,0],[0,omg**2]])

        stiff_part = K_mat * Matrix([xb,xe])

        odes = Matrix([xb.diff(t,2),xe.diff(t,2)]) + stiff_part +eps*Matrix([xb*(omg1**2-omg**2)+xb,xe*(omg2**2-omg**2)])+eps*Matrix([1*xe*cos(omg/10*t)+cos(omg*t),0*xe])


        odesy = cls(odes,Matrix([xb,xe]),order=2,eps=eps)#._as_msm()
        
        return odesy
    
class WeakNonLinearOscillatorMSM(MultiTimeScaleSolution):
    
    @classmethod
    def from_reference_data(cls):
        tau = Symbol('t')
        omega,p = symbols('omega p',positive=True)
        delta,eps, A = symbols('delta varepsilon A', positive=True)
        x = Function('x')(tau)

        p=1

        ode = ODESystem(odes=Matrix([x.diff(tau,2) + ((S.One/p)**2*(1)+delta*eps)*x +A*eps*cos(tau)+(eps)*x.diff(tau)]), dvars=Matrix([x]), ivar=tau, ode_order=2)


        odes = cls(ode.odes[0], ode.dvars[0], ivar=ode.ivar, omega=S.One, order=2,eps=eps)
        
        return odes
    
class CubicWeakOscillatorMSM(MultiTimeScaleSolution):
    
    @classmethod
    def from_reference_data(cls):
        tau = Symbol('t')
        omega,p = symbols('omega p',positive=True)
        delta,eps, A = symbols('delta varepsilon A', positive=True)
        x = Function('x')(tau)

        p=1

        ode = ODESystem(odes=Matrix([x.diff(tau,2) + ((S.One/p)**2*(1)+delta*eps)*x +A*eps*cos(tau)+(eps)*x.diff(tau)+x*eps**3]), dvars=Matrix([x]), ivar=tau, ode_order=2)


        odes = cls(ode.odes[0], ode.dvars[0], ivar=ode.ivar, omega=S.One, order=2,eps=eps)
        
        return odes


"""
This module provides the examples of mechanical models being practically used in calculations
"""

import importlib

#from .bridge import *
#from .principles import *
#from .trolley import *
#from .vehicle import *
#from .shaft import *
#from .pendulum import *
#from .car import *
#from .skyscraper import *
#from .engine import *
#from .disk import *
#from .tensioner import *
#from .tmd import *
#from .chair import *
#from .projectile import *
#from .tmac import *
#from .motorcoach import *
#from .offshore import *

from .bridge import BeamStructure, BeamElasticity, BeamBridge, BeamBridgeDamped, BeamBridgeTMD,  BeamBridgeDampedTMD
from .principles import *
from .trolley import SpringMassSystem, ForcedSpringMassSystem, SpringDamperMassSystem, SprungTrolley, NonLinearTrolley, TrolleyWithPendulum, DampedTrolleyWithPendulum, ForcedNonLinearTrolley, ForcedTrolleyWithSpring, VariableMassTrolleyWithPendulum, VariableMassTrolleyWithPendulumRayleighDamping, TrolleyWithTMD, VariableMassTrolleyWithPendulumFunction#, ForcedDampedTrolleyWithSpring

from .trolleys import DDoFTwoNonLinearTrolleys, TrolleysWithSprings, ForcedTrolleysWithSprings, ForcedDampedTrolleysWithSprings, ForcedThreeTrolleysWithSprings, ForcedTrolleysWithNonLinearSprings, ForcedDisconnectedTrolleysWithSprings, ForcedDampedThreeTrolleysWithSprings,ForcedDampedDisconnectedTrolleysWithSprings

from .vehicle import UndampedVehicleSuspension, DampedVehicleSuspension, UndampedSymmetricalVehicleSuspension, QuarterOfVehicle
from .shaft import SDoFShaft, SDoFShaftHarmonicExcitation, DoubleDiskShaft, DoubleDiskShaftHarmonicExcitation,  DampedDoubleDiskShaftHarmonicExcitation, SDoFDampedShaft
from .pendulum import Pendulum, PulledPendulum, FreePendulum, KinematicallyExcitedInvertedPendulum, MDoFElasticPendulum, ForcedTriplePendulum, Winch, DDOFCoupledPendulum, DDOFLinearizedCoupledPendulum, InvertedPendulumDDoF, DampedElasticPendulum, LinearizedTriplePendulum, DDoFWinch, DampedMeasuringTool, ExcitedDampedPendulum, DampedPendulum, ExcitedPendulum, MDoFLinearizedThreePendulumsWithSprings, PendulumKinematicExct, DoublePendulum, RollingBar, ForcedRollingBar, CompoundPendulum, ForcedCompoundPendulum

from .skyscraper import skyscraper
from .engine import EngineHousing, FreeEngine,  FreeEngineWithConstrains, FreeEngineDDOF, Engine, EngineVerticalSpringGravity,  EngineConstantVelocityVerticalSpringGravity,  EngineConstantWithReactionForce,  DampedEngineVerticalSpringGravity,  DampedEngineConstantVelocityVerticalSpringGravity,  InlineEnginePerpendicularSprings,  NonLinearInlineEnginePerpendicularSpringsGravity,  NonLinearBoxerEnginePerpendicularSprings,  DampedEngine, NonlinearEngine,  StraightNonlinearEngine, EngineWithTMD,  VeeEnginePerpendicularSprings, BoxerEnginePerpendicularSprings, NonLinearVeeEnginePerpendicularSprings
from .disk import ForcedNonLinearDisk, ForcedNonLinearDiskSpring, RollingHalfDisk, ForcedRollingHalfDisk
from .disks import TwoForcedNonLinearDisks, TwoDisksWithThreeSprings, ForcedDisksWithSerialSprings, ForcedDisksWithParallelSprings, MDoFForcedSimpleDisksWithSerialSprings, MDoFForcedSimpleDisksWithParallelSprings, MDoFForcedDisksWithParallelSprings
from .tensioner import BlowerToothedBelt, DampedBlowerToothedBelt
from .tmd import TunedMassDamper, TunedMassDamperRelativeMotion

from .projectile import MissileTrajectoryAirless, MissileTrajectory, MissileTrajectoryAerodynamic
from .tmac import SDOFWinchSystem, SDOFWinchSystemTest, SDOFDrivetrainVehicleSystem

from .principles import KinematicClutchWithSprings, LagrangeIBlocksOnInclinedPlane, MaterialPointMovement

from .offshore import CompoundSystem, SDOFSemiSubmersiblePlatform, DDOFSemiSubmersiblePlatform

from .car import CarMovementPIDAdjust, GearboxEngine, CarMovementRegulatedThrottleACC, CarMovementRegulatedThrottle, CarMovementAdjustableThrottle, OwnCombustionEngine, SecondGearCarMovement, ThirdGearCarMovement, FourthGearCarMovement, FifthGearCarMovement, CarMovementConstantThrottle

#from .motorcoach import FourDOFTrolleySuspension, DDOFTrolleySuspension2
#TODO
#from .motorcoach import DDOFTrolleySuspension

#from .chair import DampedChair4DOF, DampedChairDDOF, DampedChairSimplifiedDDOF2,


#TODO
#from .engine import #163,#162 (TODO)
#from .motorcoach import FourDOFTrolleySuspension, DDOFTrolleySuspension, DDOFTrolleySuspension2
#from .principles import  LagrangeIOnMathFunction
#from .shaft import TripleShaft
#from .trolleys import DampedTrolleysWithSprings, DampedTrolleysWithSprings, ForcedTrolleysWithSprings, MDoFForcedTrolleysWithSprings, MDoFDampedTrolleysWithSprings, DoubleTrolleyWithNonlinearSprings, DoubleTrolleyDifferentWheels, MDoFDoubleTrolleyWithNonlinearSprings, SDoFDoubleTrolleyDifferentWheels, MDoFDoubleTrolleyDifferentWheels, MDoFTMD 
#from .vehicle import DampedSymmetricalVehicleSuspension, SimplifySuspension







from sympy import (Symbol, symbols, Matrix, sin, cos, diff, sqrt, S, diag, Eq, Derivative, Expr, Heaviside)
from numbers import Number
from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex

from ..elements import MaterialPoint, Damper, Spring , base_frame, base_origin, Force




class Capacitor(Spring):

    """
    Model of a Capacitor

    Creates a singular model, after inputing correct values of capacity - c and general coordinate(s), which analytically display the dynamics of displacing spring after cummulating PE.
    """
    scheme_name = 'spring.png'
    real_name = 'spring.png'

    def __init__(self, capacity, q0, q1=0,  qs=None,ivar=Symbol('t'), frame = base_frame):
        super().__init__(stiffness=1/capacity, pos1=q0, pos2=q1,l_0=0 ,  qs=qs,ivar=ivar, frame = frame)


class Inductor(MaterialPoint):
    """
    Model of a Material point with changing point of mass:
    Creates a material point of an inertial body, after inputing correct values of mass -m and general coordinates, which follows a linear motion.
    """
    scheme_name = 'material_point.png'
    real_name = 'material_point.png'

    def __init__(self, inductance, q0 , qs=None, frame=base_frame, ivar=Symbol('t')):
        super().__init__(m=inductance, pos1=q0 , qs=qs, frame=frame, ivar=ivar)

class Resistor(Damper):
    """
    Model of Resistor
    
    Creates a material point of an inertial body, after inputing correct values of mass -m and general coordinates, which follows a linear motion.
    """
    scheme_name = 'material_point.png'
    real_name = 'material_point.png'
    def __init__(self, resistance, q0,  qs=None, ivar=Symbol('t'), frame=base_frame):
        super().__init__(c=resistance, pos1=q0, pos2=0, qs=qs, ivar=Symbol('t'), frame=base_frame)
        
class VoltageSource(Force):

    """
    Model of a voltage source

    Creates a singular model, after inputing correct values of capacity - c and general coordinate(s), which analytically display the dynamics of displacing spring after cummulating PE.
    """
    scheme_name = 'voltgesrc.png'
    real_name = 'spring.png'

    def __init__(self, voltage, q0,  qs=None,ivar=Symbol('t'), frame = base_frame):
        super().__init__(force=voltage, pos1=q0, qs=qs,ivar=ivar, frame = frame)
        
class EMForce(Force):

    """
    Creates enforcement that shows the effect of EM coupling.
    """
    scheme_name = ''
    real_name = ''

    def __init__(self, voltage, q0,  qs=None,ivar=Symbol('t'), frame = base_frame):
        super().__init__(force=voltage, pos1=q0, qs=qs,ivar=ivar, frame = frame)


from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset, atan)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from ...continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect
import pandas as pd

ix = pd.IndexSlice

from .pendulum import Pendulum, PendulumKinematicExct
from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST
from .tmd import TunedMassDamperRelativeMotion
from ...utilities.components.mech import en as mech_comp
from .trolley import VariableMassTrolleyWithPendulumRayleighDamping

from pylatex.utils import italic, NoEscape

from pint import UnitRegistry
ureg = UnitRegistry()

from functools import cached_property, lru_cache

class AdaptableTunedFluidDamper(VariableMassTrolleyWithPendulumRayleighDamping):
    
    trolley_max_value = Symbol('tv_max')
    trolley_min_value = Symbol('tv_min')
    trolley_max_spec_value = Symbol('tsv_max')
    pendulum_max_value = Symbol('pv_max')
    pendulum_min_value = Symbol('pv_min')
    pendulum_max_spec_value = Symbol('psv_max')
    mainplot_tikz_data_graph = Symbol('mx_dg')
    mainplot_tikz_value_graph = Symbol('mx_vg')
    spectrum_tikz_data_graph = Symbol('sx_dg')
    spectrum_tikz_graph = Symbol('sx_vg')
    mainplot_phi_tikz_data_graph = Symbol('mp_dg')
    mainplot_phi_tikz_graph = Symbol('mp_vg')
    spectrum_phi_tikz_data_graph = Symbol('sp_dg')
    spectrum_phi_tikz_graph = Symbol('sp_vg')

    def simulation_performance(self, params_dict, parameter, param_span, t_span, ic_list):
        
        na_df = self._ode_system.subs(params_dict).numerical_analysis(parameter=parameter, param_span=param_span)
                
        simulation_result = na_df.compute_solution(t_span = t_span, ic_list = ic_list,backend='numpy')
        
        return simulation_result.droplevel(0, axis=1)
    
    def simulation_results_presentation(self, simulation_result, dynamic_variable, caption = None):
        
        trolley_max_value = Symbol('tv_max')
        trolley_min_value = Symbol('tv_min')
        trolley_max_spec_value = Symbol('tsv_max')
        pendulum_max_value = Symbol('pv_max')
        pendulum_min_value = Symbol('pv_min')
        pendulum_max_spec_value = Symbol('psv_max')
        mainplot_tikz_data_graph = Symbol('mx_dg')
        mainplot_tikz_value_graph = Symbol('mx_vg')
        spectrum_tikz_data_graph = Symbol('sx_dg')
        spectrum_tikz_graph = Symbol('sx_vg')
        mainplot_phi_tikz_data_graph = Symbol('mp_dg')
        mainplot_phi_tikz_graph = Symbol('mp_vg')
        spectrum_phi_tikz_data_graph = Symbol('sp_dg')
        spectrum_phi_tikz_graph = Symbol('sp_vg')

        display(dynamic_variable[0])
        
        trolley_max = simulation_result.loc[:, ix[:, dynamic_variable[0]]].max().values[0].round(2)
        trolley_min = simulation_result.loc[:, ix[:, dynamic_variable[0]]].min().values[0].round(2)
        trolley_max_spec = simulation_result.loc[:, ix[:, dynamic_variable[0]]].to_frequency_domain().single_sided_rms().truncate(0, 0.6).max().values[0].round(2)

        pendulum_max = simulation_result.loc[:, ix[:, dynamic_variable[1]]].max().values[0].round(2)
        pendulum_min = simulation_result.loc[:, ix[:, dynamic_variable[1]]].min().values[0].round(2)
        pendulum_max_spec = simulation_result.loc[:, ix[:, dynamic_variable[1]]].to_frequency_domain().single_sided_rms().truncate(0, 0.6).max().values[0].round(2)

        mainplot_tikz_data = simulation_result.loc[:,ix[:,dynamic_variable[0]]][list((simulation_result.loc[:,ix[:,dynamic_variable[0]]].columns))]
        
        mainplot_tikz = mainplot_tikz_data.to_pylatex_tikz(height=NoEscape(r'8cm'),width=NoEscape(r'16cm')).in_figure(caption = f"Trolley displacement for various amount of {caption}")

        spectrum_tikz_data = (simulation_result.loc[:,ix[:,dynamic_variable[0]]].to_frequency_domain().single_sided_rms( ).truncate(0, 1))[list(((simulation_result.loc[:,ix[:,dynamic_variable[0]]].to_frequency_domain().single_sided_rms( ).truncate(0, 1)).columns))]
        
        spectrum_tikz = spectrum_tikz_data.to_pylatex_tikz(height=NoEscape(r'8cm'),width=NoEscape(r'16cm')).in_figure(caption = f"Frequency spectra of trolley displacement for various amount of {caption}")

        mainplot_phi_tikz_data = simulation_result.loc[:,ix[:,dynamic_variable[1]]][list((simulation_result.loc[:,ix[:,dynamic_variable[1]]].columns))]
        
        mainplot_phi_tikz = mainplot_phi_tikz_data.to_pylatex_tikz(height=NoEscape(r'8cm'),width=NoEscape(r'16cm')).in_figure(caption = f"Pendulum angular displacement for various amount of {caption}")

        spectrum_phi_tikz_data = (simulation_result.loc[:,ix[:,dynamic_variable[1]]].to_frequency_domain().single_sided_rms( ).truncate(0, 1))[list(((simulation_result.loc[:,ix[:,dynamic_variable[1]]].to_frequency_domain().single_sided_rms( ).truncate(0, 1)).columns))]
        
        spectrum_phi_tikz = spectrum_phi_tikz_data.to_pylatex_tikz(height=NoEscape(r'8cm'),width=NoEscape(r'16cm')).in_figure(caption = f"Frequency spectra of pendulum angular displacement for various amount of {caption}")
        
        results_presentation = {
            trolley_max_value: trolley_max,
            trolley_min_value: trolley_min,
            trolley_max_spec_value: trolley_max_spec,
            pendulum_max_value: pendulum_max,
            pendulum_min_value: pendulum_min,
            pendulum_max_spec_value: pendulum_max_spec,
            mainplot_tikz_data_graph: mainplot_tikz_data,
            mainplot_tikz_value_graph: mainplot_tikz,
            spectrum_tikz_data_graph: spectrum_tikz_data,
            spectrum_tikz_graph: spectrum_tikz,
            mainplot_phi_tikz_data_graph: mainplot_phi_tikz_data,
            mainplot_phi_tikz_graph: mainplot_phi_tikz,
            spectrum_phi_tikz_data_graph: spectrum_phi_tikz_data,
            spectrum_phi_tikz_graph: spectrum_phi_tikz,
        }
        
        return results_presentation

    def investigation_of_initial_frequency_influence(self, ic_list=None, t_span=None, params_list=None, params_dict=None):

        if ic_list is None: ic_list =[0.0, 0.0, 0.0, 0.0]
        if t_span is None: t_span=np.linspace(0,100,2001)
        if params_list is None: params_list = [0.494,0.594,0.694,0.794,0.894]
        if params_dict is None:
            params_dict= {
                self.m_p: 10,
                self.m_f: 70,
                self.b: 2,
                self.F: 50,
                self.flow_coeff: 10,
                self.t_0: 30,
                self.g: 9.81,
                self.l: 6,
                self.k: 80,
                self.m_t: 80,
                self.m_tf:70,
                self.m_pf:0
            }

        parameter=self.Omega
        
        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)

    def adaptable_tuned_mass_damper_validation(self):

        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.b: 2,
            self.F: 50,
            self.flow_coeff: 10,
            self.Omega: 0.794,
            self.t_0: 30,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.t_0
        params_dict[self.t_0] = self.t_0
        
        params_dict_2 = params_dict
        params_dict_2[self.t_0] = self.t_0
        
        params_dict_2.update({self.m_t:  8.0*params_dict_2[self.m_p],
                            self.m_tf: params_dict_2[self.m_f] + 0.000000001*params_dict_2[self.t_0],
                            self.m_pf: 0.000000001*params_dict_2[self.t_0]})

        params_dict.update({self.m_t:  8.0*params_dict[self.m_p],
                            self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })
        
        param_min=110
        param_max=120
        cases_no = 1

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)
        
        ic_list = [0.0,0.0,0.0,0.0]
        
        simple_model_na_df = self.simulation_performance(params_dict = params_dict_2, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)
        
        advanced_model_na_df = self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)
        
        return pd.concat([simple_model_na_df, advanced_model_na_df], axis=1)

    
    def investigation_of_damping_coefficient_influence(self):
        
        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.b: 15,
            self.F: 50,
            self.flow_coeff: 10,
            self.t_0: 30,
            self.Omega: 0.694,#3.14,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.b

        params_dict.update({self.m_t:  8.0*params_dict[self.m_p],
                            self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })
        
        params_dict[parameter] = parameter

        param_min=4
        param_max=20
        cases_no = 5

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)
        
        ic_list = [0.0,0.0,0.0,0.0]
        
        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)

    def investigation_of_damping_coefficient_damping_capabilities(self):
        
        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.b: 10,
            self.F: 50,
            self.flow_coeff: 10,
            self.t_0: 110,
            self.Omega: 0.694,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.b

        params_dict.update({self.m_t:  8.0*params_dict[self.m_p],
                            self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })
        
        params_dict[parameter] = parameter

        param_min=4
        param_max=20
        cases_no = 5

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)
        
        ic_list = [0.0,0.0,0.0,0.0]
        
        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)
    
    def investigation_of_activation_time_influence(self):
        
        
        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.b: 10,
            self.F: 50,
            self.flow_coeff: 10,
            self.Omega: 0.694,
            self.t_0: 110,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.t_0
        
        params_dict[self.t_0] = self.t_0

        params_dict.update({self.m_t:  8.0*params_dict[self.m_p],
                            self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })
        
        param_min=-30
        param_max=90
        cases_no = 5

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)
        
        ic_list = [0.0,0.0,0.0,0.0]
        
        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)

    
    def investigation_of_trolley_mass_influence(self):

        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.b: 10,
            self.F: 50,
            self.flow_coeff: 10,
            self.Omega: 0.694,
            self.t_0: 30,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.m_t

        params_dict[parameter] = parameter

        params_dict.update({self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })

        param_min=60
        param_max=100
        cases_no = 5

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)

        ic_list = [0.0,0.0,0.0,0.0]

        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)

    def investigation_of_pendulum_mass_influence(self):

        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.m_t: 80,
            self.b: 10,
            self.F: 50,
            self.flow_coeff: 10,
            self.Omega: 0.694,
            self.t_0: 30,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.m_p

        params_dict[parameter] = parameter

        params_dict.update({self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })

        param_min=5
        param_max=25
        cases_no = 5

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)

        ic_list = [0.0,0.0,0.0,0.0]

        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)
    
    def investigation_of_fluid_mass_influence(self):

        params_dict= {
            self.m_p: 10,
            self.m_f: 70,
            self.m_t: 80,
            self.b: 10,
            self.F: 50,
            self.flow_coeff: 10,
            self.Omega: 0.694,
            self.t_0: 30,
            self.g: 9.81,
            self.l: 6,
            self.k: 80,
        }

        t_span=np.linspace(0,100,2001)

        parameter=self.m_f

        params_dict[parameter] = parameter

        params_dict.update({self.m_tf: params_dict[self.m_f]*((S.One/2-atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi)),
                            self.m_pf: params_dict[self.m_f]*((S.One/2+atan(params_dict[self.flow_coeff]*(self.ivar-params_dict[self.t_0]))/pi))
                           })

        param_min=50
        param_max=90
        cases_no = 5

        params_list = []
        for value in np.linspace(param_min, param_max, cases_no).round(3):
            params_list.append(value)

        ic_list = [0.0,0.0,0.0,0.0]

        return self.simulation_performance(params_dict = params_dict, parameter = parameter, param_span = params_list, t_span = t_span, ic_list = ic_list)